@@ -8,22 +8,29 @@
 # isort: skip_file
 
 __all__ = [
-    "Kiara",
-    "explain",
-    "KiaraModule",
-    # "Pipeline",
-    # "PipelineStructure",
-    # "PipelineController",
-    # "PipelineModule",
-    # "DataRegistry",
     "get_version",
+    "Kiara",
+    "KiaraAPI",
+    "KiaraModule",
+    "KiaraModuleConfig",
+    "Value",
+    "ValueMap",
+    "ValueMapSchema",
+    "ValueSchema",
 ]
+import logging
 import os
+import sys
+
+import structlog
 import typing
 
-from .context import Kiara, explain  # noqa
-from .modules import KiaraModule  # noqa
 from .utils.class_loading import KiaraEntryPointItem, find_kiara_model_classes_under
+from .context import Kiara
+from .interfaces.python_api import KiaraAPI
+from .modules import KiaraModule, ValueMapSchema, KiaraModuleConfig
+from .models.values.value import Value, ValueMap
+from .models.values.value_schema import ValueSchema
 
 try:
     builtins = __import__("__builtin__")
@@ -31,6 +38,20 @@ except ImportError:
     builtins = __import__("builtins")
 
 
+# =================================================================
+# global init stuff
+
+# default logging, unless set somewhere else
+structlog.configure(
+    wrapper_class=structlog.make_filtering_bound_logger(logging.WARNING),
+)
+# check if run in Jupyter
+if "google.colab" in sys.modules or "jupyter_client" in sys.modules:
+    from kiara.interfaces import set_console_width
+
+    set_console_width()
+
+
 try:
     from rich import inspect
     from rich import print as rich_print
@@ -41,7 +62,7 @@ try:
         *objects: typing.Any,
         sep: str = " ",
         end: str = "\n",
-        file: typing.Optional[typing.IO[str]] = None,
+        file: typing.Union[typing.IO[str], None] = None,
         flush: bool = False,
     ):
 
@@ -59,7 +80,7 @@ try:
         *objects: typing.Any,
         sep: str = " ",
         end: str = "\n",
-        file: typing.Optional[typing.IO[str]] = None,
+        file: typing.Union[typing.IO[str], None] = None,
         flush: bool = False,
     ):
 
@@ -106,6 +127,10 @@ find_model_classes: KiaraEntryPointItem = (
     find_kiara_model_classes_under,
     "kiara.models",
 )
+find_model_classes_api: KiaraEntryPointItem = (
+    find_kiara_model_classes_under,
+    "kiara.interfaces.python_api.models",
+)
 
 
 def get_version() -> str:
