@@ -10,32 +10,27 @@ import structlog
 import uuid
 from alembic import command  # type: ignore
 from pydantic import Field
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    Iterable,
-    List,
-    Mapping,
-    Optional,
-    Set,
-    Type,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Set, Type, Union
 
 from kiara.context.config import KiaraConfig, KiaraContextConfig, KiaraRuntimeConfig
 from kiara.data_types import DataType
 from kiara.defaults import KIARA_DB_MIGRATIONS_CONFIG, KIARA_DB_MIGRATIONS_FOLDER
 from kiara.interfaces import get_console
+from kiara.interfaces.python_api.models.info import (
+    DataTypeClassesInfo,
+    InfoItemGroup,
+    ItemInfo,
+    KiaraModelClassesInfo,
+    ModuleTypeInfo,
+    ModuleTypesInfo,
+    OperationGroupInfo,
+    OperationTypeClassesInfo,
+)
 from kiara.interfaces.python_api.value import StoreValueResult, StoreValuesResult
 from kiara.models import KiaraModel
 from kiara.models.context import ContextSummary
-from kiara.models.info import InfoModelGroup, ItemInfo, KiaraModelClassesInfo
-from kiara.models.module import KiaraModuleTypeInfo, ModuleTypeClassesInfo
 from kiara.models.module.manifest import Manifest
-from kiara.models.module.operation import OperationGroupInfo, OperationTypeClassesInfo
 from kiara.models.runtime_environment import RuntimeEnvironment
-from kiara.models.values.data_type import DataTypeClassesInfo
 from kiara.models.values.value import ValueMap
 from kiara.registries import KiaraArchive
 from kiara.registries.aliases import AliasRegistry
@@ -51,7 +46,7 @@ from kiara.registries.modules import ModuleRegistry
 from kiara.registries.operations import OperationRegistry
 from kiara.registries.types import TypeRegistry
 from kiara.registries.workflows import WorkflowRegistry
-from kiara.utils import is_debug, log_message
+from kiara.utils import log_exception, log_message
 from kiara.utils.class_loading import find_all_archive_types
 from kiara.utils.operations import filter_operations
 
@@ -62,14 +57,16 @@ if TYPE_CHECKING:
 logger = structlog.getLogger()
 
 
-def explain(item: Any):
+def explain(item: Any, kiara: Union[None, "Kiara"] = None):
     """Pretty print information about an item on the terminal."""
 
     if isinstance(item, type):
         from kiara.modules import KiaraModule
 
         if issubclass(item, KiaraModule):
-            item = KiaraModuleTypeInfo.create_from_type_class(type_cls=item)
+            if kiara is None:
+                kiara = Kiara.instance()
+            item = ModuleTypeInfo.create_from_type_class(type_cls=item, kiara=kiara)
 
     console = get_console()
     console.print(item)
@@ -103,7 +100,7 @@ class Kiara(object):
     @classmethod
     def instance(
         cls,
-        context_name: Optional[str] = None,
+        context_name: Union[str, None] = None,
         runtime_config: Union[None, Mapping[str, Any], KiaraRuntimeConfig] = None,
     ) -> "Kiara":
         """The default *kiara* context. In most cases, it's recommended you create and manage your own, though."""
@@ -132,11 +129,11 @@ class Kiara(object):
 
     def __init__(
         self,
-        config: Optional[KiaraContextConfig] = None,
-        runtime_config: Optional[KiaraRuntimeConfig] = None,
+        config: Union[KiaraContextConfig, None] = None,
+        runtime_config: Union[KiaraRuntimeConfig, None] = None,
     ):
 
-        kc: Optional[KiaraConfig] = None
+        kc: Union[KiaraConfig, None] = None
         if not config:
             kc = KiaraConfig()
             config = kc.get_context_config()
@@ -157,7 +154,7 @@ class Kiara(object):
         self._type_registry: TypeRegistry = TypeRegistry(self)
         self._data_registry: DataRegistry = DataRegistry(kiara=self)
         self._job_registry: JobRegistry = JobRegistry(kiara=self)
-        self._module_registry: ModuleRegistry = ModuleRegistry()
+        self._module_registry: ModuleRegistry = ModuleRegistry(kiara=self)
         self._operation_registry: OperationRegistry = OperationRegistry(kiara=self)
 
         self._kiara_model_registry: ModelRegistry = ModelRegistry.instance()
@@ -167,14 +164,14 @@ class Kiara(object):
 
         self._workflow_registry: WorkflowRegistry = WorkflowRegistry(kiara=self)
 
-        self._env_mgmt: Optional[EnvironmentRegistry] = None
+        self._env_mgmt: Union[EnvironmentRegistry, None] = None
 
         metadata_augmenter = CreateMetadataDestinies(kiara=self)
         self._event_registry.add_listener(
             metadata_augmenter, *metadata_augmenter.supported_event_types()
         )
 
-        self._context_info: Optional[KiaraContextInfo] = None
+        self._context_info: Union[KiaraContextInfo, None] = None
 
         # initialize stores
         self._archive_types = find_all_archive_types()
@@ -323,7 +320,7 @@ class Kiara(object):
     # kiara session API methods
 
     def create_manifest(
-        self, module_or_operation: str, config: Optional[Mapping[str, Any]] = None
+        self, module_or_operation: str, config: Union[Mapping[str, Any], None] = None
     ) -> Manifest:
 
         if config is None:
@@ -408,17 +405,20 @@ class Kiara(object):
                 if field_aliases:
                     self.alias_registry.register_aliases(value.value_id, *field_aliases)
 
-                stored[field_name] = StoreValueResult.construct(
-                    value=value, aliases=sorted(field_aliases), error=None
+                stored[field_name] = StoreValueResult(
+                    value=value,
+                    aliases=sorted(field_aliases),
+                    error=None,
+                    persisted_data=None,
                 )
 
             except Exception as e:
-                if is_debug():
-                    import traceback
-
-                    traceback.print_exc()
-                stored[field_name] = StoreValueResult.construct(
-                    value=value, aliases=sorted(field_aliases), error=str(e)
+                log_exception(e)
+                stored[field_name] = StoreValueResult(
+                    value=value,
+                    aliases=sorted(field_aliases),
+                    error=str(e),
+                    persisted_data=None,
                 )
 
         return StoreValuesResult.construct(__root__=stored)
@@ -439,6 +439,8 @@ class Kiara(object):
             result.setdefault(archive, set()).add(alias)
         for alias, archive in self.job_registry.job_archives.items():
             result.setdefault(archive, set()).add(alias)
+        for alias, archive in self.workflow_registry.workflow_archives.items():
+            result.setdefault(archive, set()).add(alias)
 
         return result
 
@@ -446,7 +448,7 @@ class Kiara(object):
 class KiaraContextInfo(KiaraModel):
     @classmethod
     def create_from_kiara_instance(
-        cls, kiara: "Kiara", package_filter: Optional[str] = None
+        cls, kiara: "Kiara", package_filter: Union[str, None] = None
     ):
 
         data_types = kiara.type_registry.get_context_metadata(
@@ -484,11 +486,11 @@ class KiaraContextInfo(KiaraModel):
         )
 
     kiara_id: uuid.UUID = Field(description="The id of the kiara context.")
-    package_filter: Optional[str] = Field(
+    package_filter: Union[str, None] = Field(
         description="Whether this context is filtered to only include information included in a specific Python package."
     )
     data_types: DataTypeClassesInfo = Field(description="The included data types.")
-    module_types: ModuleTypeClassesInfo = Field(
+    module_types: ModuleTypesInfo = Field(
         description="The included kiara module types."
     )
     kiara_model_types: KiaraModelClassesInfo = Field(
@@ -514,7 +516,7 @@ class KiaraContextInfo(KiaraModel):
     def get_info(self, item_type: str, item_id: str) -> ItemInfo:
 
         if "data_type" == item_type or "data_types" == item_type:
-            group_info: InfoModelGroup = self.data_types
+            group_info: InfoItemGroup = self.data_types
         elif "module" in item_type:
             group_info = self.module_types
         # elif "metadata" in item_type:
@@ -538,9 +540,9 @@ class KiaraContextInfo(KiaraModel):
             )
         return group_info[item_id]
 
-    def get_all_info(self, skip_empty_types: bool = True) -> Dict[str, InfoModelGroup]:
+    def get_all_info(self, skip_empty_types: bool = True) -> Dict[str, InfoItemGroup]:
 
-        result: Dict[str, InfoModelGroup] = {}
+        result: Dict[str, InfoItemGroup] = {}
         if self.data_types or not skip_empty_types:
             result["data_types"] = self.data_types
         if self.module_types or not skip_empty_types:
