@@ -16,7 +16,7 @@ from pydantic.config import Extra
 from pydantic.env_settings import BaseSettings
 from pydantic.fields import Field, PrivateAttr
 from ruamel import yaml as r_yaml
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Optional, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Union
 
 from kiara.defaults import (
     DEFAULT_ALIAS_STORE_MARKER,
@@ -32,7 +32,7 @@ from kiara.defaults import (
 )
 from kiara.registries.environment import EnvironmentRegistry
 from kiara.registries.ids import ID_REGISTRY
-from kiara.utils import get_data_from_file
+from kiara.utils.files import get_data_from_file
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
@@ -82,7 +82,7 @@ class KiaraContextConfig(BaseModel):
         description="Paths to local folders that contain kiara pipelines.",
         default_factory=list,
     )
-    _context_config_path: Optional[Path] = PrivateAttr(default=None)
+    _context_config_path: Union[Path, None] = PrivateAttr(default=None)
 
     def add_pipelines(self, *pipelines: str):
 
@@ -159,7 +159,7 @@ class KiaraConfig(BaseSettings):
         return config
 
     @classmethod
-    def load_from_file(cls, path: Optional[Path] = None) -> "KiaraConfig":
+    def load_from_file(cls, path: Union[Path, None] = None) -> "KiaraConfig":
 
         if path is None:
             path = Path(KIARA_MAIN_CONFIG_FILE)
@@ -179,7 +179,9 @@ class KiaraConfig(BaseSettings):
         with path.open("rt") as f:
             data = yaml.load(f)
 
-        return KiaraConfig(**data)
+        config = KiaraConfig(**data)
+        config._config_path = path
+        return config
 
     context_search_paths: List[str] = Field(
         description="The base path to look for contexts in.",
@@ -207,7 +209,7 @@ class KiaraConfig(BaseSettings):
     _contexts: Dict[uuid.UUID, "Kiara"] = PrivateAttr(default_factory=dict)
     _available_context_files: Dict[str, Path] = PrivateAttr(default=None)
     _context_data: Dict[str, KiaraContextConfig] = PrivateAttr(default_factory=dict)
-    _config_path: Optional[Path] = PrivateAttr(default=None)
+    _config_path: Union[Path, None] = PrivateAttr(default=None)
 
     @validator("context_search_paths")
     def validate_context_search_paths(cls, v):
@@ -263,7 +265,9 @@ class KiaraConfig(BaseSettings):
         return {a: self.get_context_config(a) for a in self.available_context_names}
 
     def get_context_config(
-        self, context_name: Optional[str] = None, auto_generate: Optional[bool] = None
+        self,
+        context_name: Union[str, None] = None,
+        auto_generate: Union[bool, None] = None,
     ) -> KiaraContextConfig:
 
         if auto_generate is None:
@@ -321,7 +325,7 @@ class KiaraConfig(BaseSettings):
         changed = False
         if DEFAULT_DATA_STORE_MARKER not in context_config.archives.keys():
             data_store_type = "filesystem_data_store"
-            assert data_store_type in available_archives.keys()
+            assert data_store_type in available_archives.item_infos.keys()
 
             data_store_id = ID_REGISTRY.generate(comment="default data store id")
             data_archive_config = {
@@ -342,7 +346,7 @@ class KiaraConfig(BaseSettings):
 
         if DEFAULT_JOB_STORE_MARKER not in context_config.archives.keys():
             job_store_type = "filesystem_job_store"
-            assert job_store_type in available_archives.keys()
+            assert job_store_type in available_archives.item_infos.keys()
 
             job_store_id = ID_REGISTRY.generate(comment="default job store id")
             job_archive_config = {
@@ -364,7 +368,7 @@ class KiaraConfig(BaseSettings):
         if DEFAULT_ALIAS_STORE_MARKER not in context_config.archives.keys():
 
             alias_store_type = "filesystem_alias_store"
-            assert alias_store_type in available_archives.keys()
+            assert alias_store_type in available_archives.item_infos.keys()
             alias_store_id = ID_REGISTRY.generate(comment="default alias store id")
             alias_store_config = {
                 "archive_path": os.path.abspath(
@@ -385,7 +389,7 @@ class KiaraConfig(BaseSettings):
         if DEFAULT_WORKFLOW_STORE_MARKER not in context_config.archives.keys():
 
             workflow_store_type = "filesystem_workflow_store"
-            assert workflow_store_type in available_archives.keys()
+            assert workflow_store_type in available_archives.item_infos.keys()
             workflow_store_id = ID_REGISTRY.generate(
                 comment="default workflow store id"
             )
@@ -409,7 +413,7 @@ class KiaraConfig(BaseSettings):
 
         if METADATA_DESTINY_STORE_MARKER not in context_config.archives.keys():
             destiny_store_type = "filesystem_destiny_store"
-            assert destiny_store_type in available_archives.keys()
+            assert destiny_store_type in available_archives.item_infos.keys()
             destiny_store_id = ID_REGISTRY.generate(comment="default destiny store id")
             destiny_store_config = {
                 "archive_path": os.path.abspath(
@@ -430,7 +434,7 @@ class KiaraConfig(BaseSettings):
         return changed
 
     def create_context_config(
-        self, context_alias: Optional[str] = None
+        self, context_alias: Union[str, None] = None
     ) -> KiaraContextConfig:
 
         if not context_alias:
@@ -520,7 +524,7 @@ class KiaraConfig(BaseSettings):
     def find_context_config(self, context_id: uuid.UUID) -> KiaraContextConfig:
         raise NotImplementedError()
 
-    def save(self, path: Optional[Path] = None):
+    def save(self, path: Union[Path, None] = None):
         if path is None:
             path = Path(KIARA_MAIN_CONFIG_FILE)
 
@@ -549,7 +553,7 @@ class KiaraConfig(BaseSettings):
         self._config_path = path
 
     def delete(
-        self, context_name: Optional[str] = None, dry_run: bool = True
+        self, context_name: Union[str, None] = None, dry_run: bool = True
     ) -> "ContextSummary":
 
         if context_name is None:
@@ -578,6 +582,13 @@ class KiaraConfig(BaseSettings):
 
         return context_summary
 
+    def create_renderable(self, **render_config: Any):
+        from kiara.utils.output import create_recursive_table_from_model_object
+
+        return create_recursive_table_from_model_object(
+            self, render_config=render_config
+        )
+
 
 # class KiaraCurrentContextConfig(KiaraBaseConfig):
 #     """Configuration that holds the currently active context, as well as references to other available contexts."""
