@@ -5,17 +5,16 @@
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 import uuid
-from rich import box
-from rich.console import RenderableType
-from rich.table import Table
-from typing import TYPE_CHECKING, Any, Iterable, List, Mapping, Optional, Type, Union
+from typing import TYPE_CHECKING, Any, Iterable, List, Mapping, Type, Union
 
 if TYPE_CHECKING:
-    from kiara import KiaraModule
+    from rich.table import Table
+
     from kiara.data_types import DataType
     from kiara.models.module.jobs import ActiveJob
     from kiara.models.module.manifest import Manifest
     from kiara.models.values.value import Value
+    from kiara.modules import KiaraModule
 
 
 class KiaraException(Exception):
@@ -28,13 +27,13 @@ class KiaraModuleConfigException(Exception):
         msg: str,
         module_cls: Type["KiaraModule"],
         config: Mapping[str, Any],
-        parent: Optional[Exception] = None,
+        parent: Union[Exception, None] = None,
     ):
 
         self._module_cls = module_cls
         self._config = config
 
-        self._parent: Optional[Exception] = parent
+        self._parent: Union[Exception, None] = parent
 
         if not msg.endswith("."):
             _msg = msg + "."
@@ -50,13 +49,13 @@ class ValueTypeConfigException(Exception):
         msg: str,
         type_cls: Type["DataType"],
         config: Mapping[str, Any],
-        parent: Optional[Exception] = None,
+        parent: Union[Exception, None] = None,
     ):
 
         self._type_cls = type_cls
         self._config = config
 
-        self._parent: Optional[Exception] = parent
+        self._parent: Union[Exception, None] = parent
 
         if not msg.endswith("."):
             _msg = msg + "."
@@ -89,7 +88,7 @@ class NoSuchExecutionTargetException(Exception):
         self,
         selected_target: str,
         available_targets: Iterable[str],
-        msg: Optional[str] = None,
+        msg: Union[str, None] = None,
     ):
 
         if msg is None:
@@ -103,13 +102,13 @@ class KiaraProcessingException(Exception):
     def __init__(
         self,
         msg: Union[str, Exception],
-        module: Optional["KiaraModule"] = None,
-        inputs: Optional[Mapping[str, "Value"]] = None,
+        module: Union["KiaraModule", None] = None,
+        inputs: Union[Mapping[str, "Value"], None] = None,
     ):
-        self._module: Optional["KiaraModule"] = module
-        self._inputs: Optional[Mapping[str, Value]] = inputs
+        self._module: Union["KiaraModule", None] = module
+        self._inputs: Union[Mapping[str, Value], None] = inputs
         if isinstance(msg, Exception):
-            self._parent: Optional[Exception] = msg
+            self._parent: Union[Exception, None] = msg
             _msg = str(msg)
         else:
             self._parent = None
@@ -125,7 +124,7 @@ class KiaraProcessingException(Exception):
         return self._inputs  # type: ignore
 
     @property
-    def parent_exception(self) -> Optional[Exception]:
+    def parent_exception(self) -> Union[Exception, None]:
         return self._parent
 
 
@@ -150,7 +149,7 @@ class InvalidValuesException(Exception):
                     msg_parts.append(f"{k}: {v}")
                 _msg = f"Invalid values: {', '.join(msg_parts)}"
         elif isinstance(msg, Exception):
-            self._parent: Optional[Exception] = msg
+            self._parent: Union[Exception, None] = msg
             _msg = str(msg)
         else:
             self._parent = None
@@ -158,7 +157,11 @@ class InvalidValuesException(Exception):
 
         super().__init__(_msg)
 
-    def create_renderable(self, **config: Any) -> Table:
+    def create_renderable(self, **config: Any) -> "Table":
+
+        from rich import box
+        from rich.console import RenderableType
+        from rich.table import Table
 
         table = Table(box=box.SIMPLE, show_header=True)
 
@@ -186,7 +189,7 @@ class JobConfigException(Exception):
         self._inputs: Mapping[str, Any] = inputs
 
         if isinstance(msg, Exception):
-            self._parent: Optional[Exception] = msg
+            self._parent: Union[Exception, None] = msg
             _msg = str(msg)
         else:
             self._parent = None
@@ -204,13 +207,33 @@ class JobConfigException(Exception):
 
 
 class FailedJobException(Exception):
-    def __init__(self, job: "ActiveJob", msg: Optional[str] = None):
+    def __init__(self, job: "ActiveJob", msg: Union[str, None] = None):
 
         self.job: ActiveJob = job
         if msg is None:
             msg = "Job failed."
+        self.msg = msg
         super().__init__(msg)
 
+    def create_renderable(self, **config: Any):
+
+        from rich import box
+        from rich.console import Group
+        from rich.panel import Panel
+        from rich.table import Table
+
+        table = Table(show_header=False, box=box.SIMPLE)
+        table.add_column("key", style="i")
+        table.add_column("value")
+
+        table.add_row("job_id", str(self.job.job_id))
+        table.add_row("module_type", self.job.job_config.module_type)
+
+        group = Group(
+            Panel(f"[red]Error[/red]: [i]{self.msg}[i]", box=box.SIMPLE), table
+        )
+        return group
+
 
 class NoSuchValueException(Exception):
 
@@ -218,7 +241,7 @@ class NoSuchValueException(Exception):
 
 
 class NoSuchValueIdException(NoSuchValueException):
-    def __init__(self, value_id: uuid.UUID, msg: Optional[str] = None):
+    def __init__(self, value_id: uuid.UUID, msg: Union[str, None] = None):
         self.value_id: uuid.UUID
         if not msg:
             msg = f"No value with id: {value_id}."
@@ -226,8 +249,53 @@ class NoSuchValueIdException(NoSuchValueException):
 
 
 class NoSuchValueAliasException(NoSuchValueException):
-    def __init__(self, alias: str, msg: Optional[str] = None):
+    def __init__(self, alias: str, msg: Union[str, None] = None):
         self.value_id: uuid.UUID
         if not msg:
             msg = f"No value with alias: {alias}."
         super().__init__(msg)
+
+
+class NoSuchWorkflowException(Exception):
+    def __init__(self, workflow: Union[uuid.UUID, str], msg: Union[str, None] = None):
+        self._workflow: Union[str, uuid.UUID] = workflow
+        if not msg:
+            msg = f"No such workflow: {workflow}"
+        super().__init__(msg)
+
+    @property
+    def alias_requested(self) -> bool:
+
+        if isinstance(self._workflow, str):
+            try:
+                uuid.UUID(self._workflow)
+                return False
+            except Exception:
+                return True
+        else:
+            return False
+
+
+class NoSuchOperationException(Exception):
+    def __init__(
+        self,
+        operation_id: str,
+        available_operations: Iterable[str],
+        msg: Union[None, str] = None,
+    ):
+
+        self._operation_id: str = operation_id
+        self._available_operations: Iterable[str] = available_operations
+
+        if not msg:
+            msg = f"No operation with id: {operation_id} available."
+
+        super().__init__(msg)
+
+    @property
+    def available_operations(self) -> Iterable[str]:
+        return self._available_operations
+
+    @property
+    def operation_id(self) -> str:
+        return self._operation_id
