@@ -14,11 +14,12 @@ import rich_click as click
 import structlog
 import sys
 from pathlib import Path
-from typing import Optional, Tuple
+from typing import Tuple, Union
 
 from kiara.context.config import KiaraConfig
 from kiara.defaults import KIARA_CONFIG_FILE_NAME, KIARA_MAIN_CONFIG_FILE
-from kiara.utils import is_debug, is_develop
+from kiara.interfaces.python_api import KiaraAPI
+from kiara.utils import is_debug
 from kiara.utils.class_loading import find_all_cli_subcommands
 from kiara.utils.cli import terminal_print
 
@@ -27,14 +28,6 @@ from kiara.utils.cli import terminal_print
 # click.rich_click.USE_MARKDOWN = True
 click.rich_click.USE_RICH_MARKUP = True
 
-# try:
-#     import uvloop
-#
-#     uvloop.install()
-# except Exception:
-#     pass
-
-# click.anyio_backend = "asyncio"
 
 if is_debug():
     structlog.configure(
@@ -70,7 +63,9 @@ CLICK_CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
     required=False,
 )
 @click.pass_context
-def cli(ctx, config: Optional[str], context: Optional[str], pipelines: Tuple[str]):
+def cli(
+    ctx, config: Union[str, None], context: Union[str, None], pipelines: Tuple[str]
+):
     """[i b]kiara[/b i] ia a data-orchestration framework; this is the command-line frontend for it.
 
 
@@ -125,13 +120,12 @@ def cli(ctx, config: Optional[str], context: Optional[str], pipelines: Tuple[str
 
     kiara = kiara_config.create_context(context=context, extra_pipelines=pipelines)
     ctx.obj["kiara"] = kiara
+    ctx.obj["kiara_api"] = KiaraAPI(kiara=kiara)
     ctx.obj["kiara_config"] = kiara_config
     ctx.obj["kiara_context_name"] = context
 
 
 for plugin in find_all_cli_subcommands():
-    if plugin.name == "dev" and not is_develop():
-        continue
     cli.add_command(plugin)
 
 if __name__ == "__main__":
