@@ -19,14 +19,19 @@ from rich.console import Console, ConsoleOptions, RenderableType, RenderResult
 from rich.jupyter import JupyterMixin
 from rich.panel import Panel
 from rich.table import Table
-from typing import Any, ClassVar, Dict, Iterable, List, Optional
+from rich.tree import Tree
+from typing import Any, ClassVar, Dict, Iterable, List, Mapping, Union
 
 from kiara.defaults import KIARA_HASH_FUNCTION
-from kiara.utils import orjson_dumps
+from kiara.registries.templates import TemplateRegistry
 from kiara.utils.class_loading import _default_id_func
+from kiara.utils.develop import log_dev_message
 from kiara.utils.hashing import compute_cid
+from kiara.utils.html import generate_html
+from kiara.utils.json import orjson_dumps
 from kiara.utils.models import (
-    assemble_subcomponent_tree,
+    assemble_subcomponent_graph,
+    create_subcomponent_tree_renderable,
     get_subcomponent_from_model,
     retrieve_data_subcomponent_keys,
 )
@@ -41,12 +46,15 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
 
     __slots__ = ["__weakref__"]
 
-    class Config:
+    class Config(object):
         json_loads = orjson.loads
         json_dumps = orjson_dumps
         extra = Extra.forbid
 
-        # allow_mutation = False
+    # @classmethod
+    # def get_model_title(cls):
+    #
+    #     return to_camel_case(cls._kiara_model_name)
 
     @classmethod
     def get_schema_hash(cls) -> int:
@@ -58,15 +66,15 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         cls._schema_hash_cache = h[obj]
         return cls._schema_hash_cache
 
-    _graph_cache: Optional[nx.DiGraph] = PrivateAttr(default=None)
-    _subcomponent_names_cache: Optional[List[str]] = PrivateAttr(default=None)
+    _graph_cache: Union[nx.DiGraph, None] = PrivateAttr(default=None)
+    _subcomponent_names_cache: Union[List[str], None] = PrivateAttr(default=None)
     _dynamic_subcomponents: Dict[str, "KiaraModel"] = PrivateAttr(default_factory=dict)
-    _id_cache: Optional[str] = PrivateAttr(default=None)
-    _category_id_cache: Optional[str] = PrivateAttr(default=None)
+    _id_cache: Union[str, None] = PrivateAttr(default=None)
+    _category_id_cache: Union[str, None] = PrivateAttr(default=None)
     _schema_hash_cache: ClassVar = None
-    _cid_cache: Optional[CID] = PrivateAttr(default=None)
-    _dag_cache: Optional[bytes] = PrivateAttr(default=None)
-    _size_cache: Optional[int] = PrivateAttr(default=None)
+    _cid_cache: Union[CID, None] = PrivateAttr(default=None)
+    _dag_cache: Union[bytes, None] = PrivateAttr(default=None)
+    _size_cache: Union[int, None] = PrivateAttr(default=None)
 
     def _retrieve_data_to_hash(self) -> EncodableType:
         """Return data important for hashing this model instance. Implemented by sub-classes.
@@ -142,13 +150,13 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         return self._subcomponent_names_cache
 
     @property
-    def subcomponent_tree(self) -> Optional[nx.DiGraph]:
+    def subcomponent_tree(self) -> Union[nx.DiGraph, None]:
         """A tree structure, containing all sub-components (and their subcomponents) of this model."""
         if not self.subcomponent_keys:
             return None
 
         if self._graph_cache is None:
-            self._graph_cache = assemble_subcomponent_tree(self)
+            self._graph_cache = assemble_subcomponent_graph(self)
         return self._graph_cache
 
     def get_subcomponent(self, path: str) -> "KiaraModel":
@@ -200,6 +208,36 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
         rend = self.create_renderable(**config)
         return Panel(rend, box=box.ROUNDED, title=title, title_align="left")
 
+    def create_html(self, **config) -> str:
+
+        template_registry = TemplateRegistry.instance()
+        template = template_registry.get_template_for_model_type(
+            model_type=self.model_type_id, template_format="html"
+        )
+
+        if template:
+            try:
+                result = template.render(instance=self)
+                return result
+            except Exception as e:
+                log_dev_message(
+                    title="html-rendering error",
+                    msg=f"Failed to render html for model '{self.instance_id}' type '{self.model_type_id}': {e}",
+                )
+
+        try:
+            html = generate_html(item=self, add_header=False)
+            return html
+        except Exception as e:
+            log_dev_message(
+                title="html-generation error",
+                msg=f"Failed to generate html for model '{self.instance_id}' type '{self.model_type_id}': {e}",
+            )
+
+        r = self.create_renderable(**config)
+        mime_bundle = r._repr_mimebundle_(include=[], exclude=[])  # type: ignore
+        return mime_bundle["text/html"]
+
     def create_renderable(self, **config: Any) -> RenderableType:
 
         from kiara.utils.output import extract_renderable
@@ -217,17 +255,24 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
             table.add_row(k, v)
         return table
 
-    def create_html(self, **config) -> str:
+    def create_renderable_tree(self, **config: Any) -> Tree:
 
-        r = self.create_renderable()
-        if hasattr(r, "_repr_mimebundle_"):
-            mime_bundle = r._repr_mimebundle_(include=[], exclude=[])  # type: ignore
-        else:
-            raise NotImplementedError(
-                f"Type '{self.__class__}' can't be rendered as html (yet)."
-            )
+        show_data = config.get("show_data", False)
+        tree = create_subcomponent_tree_renderable(data=self, show_data=show_data)
+        return tree
 
-        return mime_bundle["text/html"]
+    def create_info_data(self, **config) -> Mapping[str, Any]:
+
+        include = config.get("include", None)
+
+        result = {}
+        for k in self.__fields__.keys():
+            if include is not None and k not in include:
+                continue
+            attr = getattr(self, k)
+            v = attr
+            result[k] = v
+        return result
 
     def as_dict_with_schema(self) -> Dict[str, Dict[str, Any]]:
         return {"data": self.dict(), "schema": self.schema()}
@@ -263,8 +308,29 @@ class KiaraModel(ABC, BaseModel, JupyterMixin):
     def __str__(self):
         return self.__repr__()
 
+    def _repr_html_(self):
+        return self.create_html()
+
     def __rich_console__(
         self, console: Console, options: ConsoleOptions
     ) -> RenderResult:
 
         yield self.create_renderable()
+
+    # def _repr_mimebundle_(
+    #     self: "ConsoleRenderable",
+    #     include: Sequence[str],
+    #     exclude: Sequence[str],
+    #     **kwargs: Any,
+    # ) -> Dict[str, str]:
+    #
+    #     console = get_console()
+    #     segments = list(console.render(self, console.options))
+    #     html = _render_segments(segments)
+    #     text = console._render_buffer(segments)
+    #     data = {"text/plain": text, "text/html": html}
+    #     if include:
+    #         data = {k: v for (k, v) in data.items() if k in include}
+    #     if exclude:
+    #         data = {k: v for (k, v) in data.items() if k not in exclude}
+    #     return data
