@@ -3,15 +3,15 @@
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
-
+import copy
 import datetime
 import structlog
 import uuid
 from pydantic import Field, PrivateAttr
 from rich.tree import Tree
-from typing import TYPE_CHECKING, Any, Dict, Mapping, Optional
+from typing import TYPE_CHECKING, Any, Dict, Mapping, Union
 
-from kiara.defaults import NONE_VALUE_ID
+from kiara.defaults import NONE_VALUE_ID, SpecialValue
 from kiara.models.values.value import Value, ValueMap
 from kiara.models.values.value_schema import ValueSchema
 from kiara.utils.cli import terminal_print
@@ -26,18 +26,19 @@ VALUE_ALIAS_SEPARATOR = "."
 
 
 class AliasValueMap(ValueMap):
+    """A model class that holds a tree of values and their schemas."""
 
     _kiara_model_id = "instance.value_map.aliases"
 
-    alias: Optional[str] = Field(description="This maps own (full) alias.")
+    alias: Union[str, None] = Field(description="This maps own (full) alias.")
     version: int = Field(description="The version of this map (in this maps parent).")
-    created: Optional[datetime.datetime] = Field(
-        description="The time this map was created."
+    created: Union[datetime.datetime, None] = Field(
+        description="The time this map was created.", default=None
     )
-    assoc_schema: Optional[ValueSchema] = Field(
+    assoc_schema: Union[ValueSchema, None] = Field(
         description="The schema for this maps associated value."
     )
-    assoc_value: Optional[uuid.UUID] = Field(
+    assoc_value: Union[uuid.UUID, None] = Field(
         description="The value that is associated with this map."
     )
 
@@ -58,8 +59,8 @@ class AliasValueMap(ValueMap):
         return self._is_stored
 
     def get_child_map(
-        self, field_name: str, version: Optional[str] = None
-    ) -> Optional["AliasValueMap"]:
+        self, field_name: str, version: Union[str, None] = None
+    ) -> Union["AliasValueMap", None]:
         """Get the child map for the specified field / version combination.
 
         Raises an error if the child field does not exist. Returns 'None' if not value is set yet (but schema is).
@@ -71,9 +72,11 @@ class AliasValueMap(ValueMap):
         if VALUE_ALIAS_SEPARATOR not in field_name:
 
             if self.values_schema.get(field_name, None) is None:
-                raise KeyError(
-                    f"No field name '{field_name}'. Available fields: {', '.join(self.values_schema.keys())}"
-                )
+                if not self.values_schema:
+                    msg = "No available fields"
+                else:
+                    msg = "Available fields: " + ", ".join(self.values_schema.keys())
+                raise KeyError(f"No field name '{field_name}'. {msg}")
 
             field_items = self.value_items[field_name]
             if not field_items:
@@ -102,15 +105,17 @@ class AliasValueMap(ValueMap):
         if item.assoc_value is None:
             raise Exception(f"No value associated for field '{field_name}'.")
 
-        return self._data_registry.get_value(value_id=item.assoc_value)
+        return self._data_registry.get_value(value=item.assoc_value)
 
     def get_value_id(self, field_name: str) -> uuid.UUID:
 
         item = self.get_child_map(field_name=field_name)
         if item is None:
-            return NONE_VALUE_ID
+            result = NONE_VALUE_ID
         else:
-            return item.assoc_value if item.assoc_value is not None else NONE_VALUE_ID
+            result = item.assoc_value if item.assoc_value is not None else NONE_VALUE_ID
+
+        return result
 
     def get_all_value_ids(
         self,
@@ -124,13 +129,6 @@ class AliasValueMap(ValueMap):
             result[k] = v_id
         return result
 
-    def set_value(self, field_name: str, data: Any) -> None:
-
-        assert VALUE_ALIAS_SEPARATOR not in field_name
-
-        value = self._data_registry.register_data(data)
-        self.set_alias(alias=field_name, value_id=value.value_id)
-
     def set_alias_schema(self, alias: str, schema: ValueSchema):
 
         if self._schema_locked:
@@ -148,7 +146,7 @@ class AliasValueMap(ValueMap):
                 self._set_local_field_schema(
                     field_name=child, schema=ValueSchema(type="none")
                 )
-                child_map = self.set_alias(alias=child, value_id=None)
+                child_map = self._set_alias(alias=child, data=None)
 
             assert child_map is not None
 
@@ -179,7 +177,7 @@ class AliasValueMap(ValueMap):
         self.values_schema[field_name] = schema
         self.value_items[field_name] = {}
 
-    def get_alias(self, alias: str) -> Optional["AliasValueMap"]:
+    def get_alias(self, alias: str) -> Union["AliasValueMap", None]:
 
         if VALUE_ALIAS_SEPARATOR not in alias:
             if "@" in alias:
@@ -203,33 +201,54 @@ class AliasValueMap(ValueMap):
 
             return child_map.get_alias(rest)
 
-    def set_aliases(self, **aliases) -> Mapping[str, "AliasValueMap"]:
+    def set_value(self, field_name: str, data: Any) -> None:
+
+        assert VALUE_ALIAS_SEPARATOR not in field_name
+
+        self._set_alias(alias=field_name, data=data)
+
+    def _set_aliases(self, **aliases: Any) -> Mapping[str, "AliasValueMap"]:
 
         result = {}
         for k, v in aliases.items():
-            r = self.set_alias(alias=k, value_id=v)
+            r = self._set_alias(alias=k, data=v)
             result[k] = r
 
         return result
 
-    def set_alias(self, alias: str, value_id: Optional[uuid.UUID]) -> "AliasValueMap":
+    def _set_alias(self, alias: str, data: Any) -> "AliasValueMap":
 
         if VALUE_ALIAS_SEPARATOR not in alias:
-            child = None
-            field_name: Optional[str] = alias
-            rest = None
-        else:
-            child, rest = alias.split(VALUE_ALIAS_SEPARATOR, maxsplit=1)
-            field_name = None
+            field_name: Union[str, None] = alias
 
-        if child is None:
             # means we are setting the alias in this map
             assert field_name is not None
+
+            vs = self.values_schema[alias]
+            if vs.type == "none":
+                assert data is None
+                value_id = None
+            else:
+
+                if data in [None, SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
+                    if vs.default:
+                        if callable(vs.default):
+                            data = vs.default()
+                        else:
+                            data = copy.deepcopy(vs.default)
+
+                value = self._data_registry.register_data(data=data, schema=vs)
+                value_id = value.value_id
+
             new_map = self._set_local_value_item(
                 field_name=field_name, value_id=value_id
             )
             return new_map
+
         else:
+            child, rest = alias.split(VALUE_ALIAS_SEPARATOR, maxsplit=1)
+            field_name = None
+
             # means we are dealing with an intermediate alias map
             assert rest is not None
             assert child is not None
@@ -242,7 +261,7 @@ class AliasValueMap(ValueMap):
                 else:
                     self.set_alias_schema(alias=child, schema=ValueSchema(type="any"))
 
-            field_item: Optional[AliasValueMap] = None
+            field_item: Union[AliasValueMap, None] = None
             try:
                 field_item = self.get_child_map(field_name=child)
             except KeyError:
@@ -274,19 +293,19 @@ class AliasValueMap(ValueMap):
             new_map._data_registry = self._data_registry
             self.value_items[child][new_version] = new_map
 
-            new_map.set_alias(alias=rest, value_id=value_id)
+            new_map._set_alias(alias=rest, data=data)
 
         return new_map
 
     def _set_local_value_item(
-        self, field_name: str, value_id: Optional[uuid.UUID] = None
+        self, field_name: str, value_id: Union[uuid.UUID, None] = None
     ) -> "AliasValueMap":
 
         assert VALUE_ALIAS_SEPARATOR not in field_name
 
-        value: Optional[Value] = None
+        value: Union[Value, None] = None
         if value_id is not None:
-            value = self._data_registry.get_value(value_id=value_id)
+            value = self._data_registry.get_value(value=value_id)
             assert value is not None
             assert value.value_id == value_id
 
