@@ -14,37 +14,47 @@ from rich import box
 from rich.console import RenderableType
 from rich.markdown import Markdown
 from rich.table import Table
-from typing import Any, Callable, Dict, List, Mapping, Optional, Tuple, Type, Union
+from typing import Any, Callable, Dict, List, Mapping, Tuple, Type, Union
 
 from kiara.defaults import DEFAULT_NO_DESC_VALUE
 from kiara.models import KiaraModel
-from kiara.utils import merge_dicts
+from kiara.utils.dicts import merge_dicts
 from kiara.utils.global_metadata import get_metadata_for_python_module_or_class
 
 
 class AuthorModel(BaseModel):
+    """Details about an author of a resource."""
+
+    class Config:
+        title = "Author"
 
     name: str = Field(description="The full name of the author.")
-    email: Optional[EmailStr] = Field(
+    email: Union[EmailStr, None] = Field(
         description="The email address of the author", default=None
     )
 
 
 class LinkModel(BaseModel):
+    """A description and url for a reference of any kind."""
+
+    class Config:
+        title = "Link"
 
     url: AnyUrl = Field(description="The url.")
-    desc: Optional[str] = Field(
+    desc: Union[str, None] = Field(
         description="A short description of the link content.",
         default=DEFAULT_NO_DESC_VALUE,
     )
 
 
 class AuthorsMetadataModel(KiaraModel):
+    """Information about all authors of a resource."""
 
     _kiara_model_id = "metadata.authors"
 
     class Config:
         extra = Extra.ignore
+        title = "Authors"
 
     _metadata_key = "origin"
 
@@ -76,11 +86,13 @@ class AuthorsMetadataModel(KiaraModel):
 
 
 class ContextMetadataModel(KiaraModel):
+    """Information about the context of a resource."""
 
     _kiara_model_id = "metadata.context"
 
     class Config:
         extra = Extra.ignore
+        title = "Context"
 
     @classmethod
     def from_class(cls, item_cls: Type):
@@ -98,7 +110,7 @@ class ContextMetadataModel(KiaraModel):
         description="A list of tags for the item.", default_factory=list
     )
     labels: Dict[str, str] = Field(
-        description="A list of labels for the item.", default_factory=list
+        description="A list of labels for the item.", default_factory=dict
     )
 
     def create_renderable(self, **config: Any) -> RenderableType:
@@ -128,7 +140,7 @@ class ContextMetadataModel(KiaraModel):
         self,
         ref_type: str,
         url: str,
-        desc: Optional[str] = None,
+        desc: Union[str, None] = None,
         force: bool = False,
     ):
 
@@ -137,7 +149,7 @@ class ContextMetadataModel(KiaraModel):
         link = LinkModel(url=url, desc=desc)
         self.references[ref_type] = link
 
-    def get_url_for_reference(self, ref: str) -> Optional[str]:
+    def get_url_for_reference(self, ref: str) -> Union[str, None]:
 
         link = self.references.get(ref, None)
         if not link:
@@ -147,6 +159,10 @@ class ContextMetadataModel(KiaraModel):
 
 
 class DocumentationMetadataModel(KiaraModel):
+    """Documentation about a resource."""
+
+    class Config:
+        title = "Documentation"
 
     _kiara_model_id = "metadata.documentation"
 
@@ -175,7 +191,7 @@ class DocumentationMetadataModel(KiaraModel):
         return cls.from_string(doc)
 
     @classmethod
-    def from_string(cls, doc: Optional[str]):
+    def from_string(cls, doc: Union[str, None]):
 
         if not doc:
             doc = DEFAULT_NO_DESC_VALUE
@@ -209,7 +225,7 @@ class DocumentationMetadataModel(KiaraModel):
             return cls(description=desc, doc=doc)
 
     @classmethod
-    def create(cls, item: Any):
+    def create(cls, item: Any = None):
 
         if not item:
             return cls.from_string(DEFAULT_NO_DESC_VALUE)
@@ -227,7 +243,7 @@ class DocumentationMetadataModel(KiaraModel):
     description: str = Field(
         description="Short description of the item.", default=DEFAULT_NO_DESC_VALUE
     )
-    doc: Optional[str] = Field(
+    doc: Union[str, None] = Field(
         description="Detailed documentation of the item (in markdown).", default=None
     )
 
