@@ -8,7 +8,9 @@
 import networkx as nx
 from functools import lru_cache
 from pydantic import Field, PrivateAttr, root_validator
-from typing import Any, Dict, Iterable, List, Mapping, Set
+from rich.console import RenderableType
+from rich.tree import Tree
+from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Set
 
 from kiara.models import KiaraModel
 from kiara.models.module.pipeline import PipelineConfig, PipelineStep
@@ -22,77 +24,102 @@ from kiara.models.module.pipeline.value_refs import (
 )
 from kiara.models.values.value_schema import ValueSchema
 
+if TYPE_CHECKING:
+    pass
+
 
 def generate_pipeline_endpoint_name(step_id: str, value_name: str):
 
     return f"{step_id}__{value_name}"
 
 
-# def calculate_shortest_field_aliases(
-#     steps: List[PipelineStep], alias_type: str, alias_for: str
-# ):
-#     """Utility method to figure out the best field aliases automatically."""
-#
-#     assert alias_for in ["inputs", "outputs"]
-#     if alias_type == "auto_all_outputs":
-#
-#         aliases: Dict[str, List[str]] = {}
-#
-#         for step in steps:
-#
-#             if alias_for == "inputs":
-#                 field_names = step.module.input_names
-#             else:
-#                 field_names = step.module.output_names
-#
-#             for field_name in field_names:
-#                 aliases.setdefault(field_name, []).append(step.step_id)
-#
-#         result = {}
-#         for field_name, step_ids in aliases.items():
-#             if len(step_ids) == 1:
-#                 result[
-#                     generate_pipeline_endpoint_name(step_ids[0], field_name)
-#                 ] = field_name
-#             else:
-#                 for step_id in step_ids:
-#                     generated = generate_pipeline_endpoint_name(step_id, field_name)
-#                     result[generated] = generated
-#
-#     elif alias_type == "auto":
-#
-#         aliases = {}
-#
-#         for stage_nr, step in enumerate(steps):
-#
-#             _field_names: Optional[Iterable[str]] = None
-#             if alias_for == "inputs":
-#                 _field_names = step.module.input_names
-#             else:
-#                 if stage_nr == len(steps) - 1:
-#                     _field_names = step.module.output_names
-#
-#             if not _field_names:
-#                 continue
-#
-#             for field_name in _field_names:
-#                 aliases.setdefault(field_name, []).append(step.step_id)
-#
-#         result = {}
-#         for field_name, step_ids in aliases.items():
-#             if len(step_ids) == 1:
-#                 result[
-#                     generate_pipeline_endpoint_name(step_ids[0], field_name)
-#                 ] = field_name
-#             else:
-#                 for step_id in step_ids:
-#                     generated = generate_pipeline_endpoint_name(step_id, field_name)
-#                     result[generated] = generated
-#
-#     return result
+class StepInfo(KiaraModel):
+
+    _kiara_model_id = "info.pipeline_step"
+
+    step: PipelineStep = Field(description="The pipeline step object.")
+    inputs: Dict[str, StepInputRef] = Field(
+        description="Reference(s) to the fields that feed this steps inputs."
+    )
+    outputs: Dict[str, StepOutputRef] = Field(
+        description="Reference(s) to the fields that are fed by this steps outputs."
+    )
+    required: bool = Field(
+        description="Whether this step is always required or whether all his outputs feed into optional input fields."
+    )
+    processing_stage: int = Field(
+        description="The index of the processing stage of this step."
+    )
+
+    @property
+    def step_id(self) -> str:
+        return self.step.step_id
 
 
-# ALLOWED_INPUT_ALIAS_MARKERS = ["auto", "auto_all_outputs"]
+class PipelineStage(KiaraModel):
+
+    _kiara_model_id = "info.pipeline_stage"
+
+    @classmethod
+    def from_pipeline_structure(
+        cls, structure: "PipelineStructure"
+    ) -> Dict[int, "PipelineStage"]:
+        used_pipeline_inputs: Set[str] = set()
+        used_pipeline_outputs: Set[str] = set()
+        result = {}
+        for idx, stage in enumerate(structure.processing_stages, start=1):
+            stage_steps = []
+            inputs = []
+            outputs = []
+
+            for step_id in stage:
+                step = structure.get_step(step_id=step_id)
+                stage_steps.append(step.step_id)
+                for pipeline_input, ref in structure.pipeline_input_refs.items():
+                    if pipeline_input in used_pipeline_inputs:
+                        continue
+                    for con_inp in ref.connected_inputs:
+                        if con_inp.step_id == step_id and pipeline_input not in inputs:
+                            inputs.append(pipeline_input)
+                for pipeline_output, out_ref in structure.pipeline_output_refs.items():
+                    if pipeline_output in used_pipeline_outputs:
+                        continue
+                    if out_ref.connected_output.step_id == step_id:
+                        outputs.append(pipeline_output)
+
+            stage_used_inputs = list(used_pipeline_inputs)
+            stage_used_outputs = list(used_pipeline_outputs)
+
+            result[idx] = PipelineStage(
+                stage_index=idx,
+                steps=stage_steps,
+                pipeline_inputs=inputs,
+                pipeline_outputs=outputs,
+                previous_inputs=stage_used_inputs,
+                previous_outputs=stage_used_outputs,
+            )
+
+            used_pipeline_inputs.update(inputs)
+            used_pipeline_outputs.update(outputs)
+
+        return result
+
+    stage_index: int = Field(description="The index of this stage.")
+    steps: List[str] = Field(
+        description="The pipeline steps that are executed in this stage."
+    )
+    pipeline_inputs: List[str] = Field(
+        description="The pipeline inputs required for this stage."
+    )
+    pipeline_outputs: List[str] = Field(
+        description="The pipeline outputs that are ready once this stage is processed."
+    )
+    previous_inputs: List[str] = Field(
+        description="Pipeline inputs that are already set by this stage."
+    )
+    previous_outputs: List[str] = Field(
+        description="Pipeline outputs that are already computed by this stage."
+    )
 
 
 class PipelineStructure(KiaraModel):
@@ -104,6 +131,7 @@ class PipelineStructure(KiaraModel):
         description="The underlying pipeline config."
     )
     steps: List[PipelineStep] = Field(description="The pipeline steps ")
+    # stages: Mapping[int, PipelineStage] = Field(description="Details about each of the pipeline stages.")
     input_aliases: Dict[str, str] = Field(description="The input aliases.")
     output_aliases: Dict[str, str] = Field(description="The output aliases.")
 
@@ -119,6 +147,8 @@ class PipelineStructure(KiaraModel):
                 "Only 'pipeline_config' key allowed when creating a pipeline structure object."
             )
 
+        if isinstance(pipeline_config, Mapping):
+            pipeline_config = PipelineConfig(**pipeline_config)
         _config: PipelineConfig = pipeline_config
         _steps: List[PipelineStep] = list(_config.steps)
 
@@ -159,6 +189,9 @@ class PipelineStructure(KiaraModel):
                 f"Invalid output reference(s): {', '.join(invalid_output_names)}. Must be one of: {', '.join(valid_output_names)}"
             )
 
+        # stages = PipelineStage.from_pipeline_structure(stages=)
+
+        # values["steps"] = {step.step_id: step for step in _steps}
         values["steps"] = _steps
         values["input_aliases"] = _input_aliases
         values["output_aliases"] = _output_aliases
@@ -174,9 +207,11 @@ class PipelineStructure(KiaraModel):
     _data_flow_graph_simple: nx.DiGraph = PrivateAttr(None)  # type: ignore
 
     _processing_stages: List[List[str]] = PrivateAttr(None)  # type: ignore
+    _stages_info: Mapping[int, PipelineStage] = PrivateAttr(None)  # type: ignore
 
     # holds details about the (current) processing steps contained in this workflow
-    _steps_details: Dict[str, Any] = PrivateAttr(None)  # type: ignore
+    _steps_details: Dict[str, StepInfo] = PrivateAttr(None)  # type: ignore
+    # _info: "PipelineStructureInfo" = PrivateAttr(None)  # type: ignore
 
     def _retrieve_data_to_hash(self) -> Any:
         return {
@@ -189,7 +224,7 @@ class PipelineStructure(KiaraModel):
         return self.pipeline_config.instance_id
 
     @property
-    def steps_details(self) -> Mapping[str, Any]:
+    def steps_details(self) -> Mapping[str, StepInfo]:
 
         if self._steps_details is None:
             self._process_steps()
@@ -219,31 +254,31 @@ class PipelineStructure(KiaraModel):
 
         d = self.steps_details.get(step_id, None)
         if d is None:
-            raise Exception(f"No module with id: {step_id}")
+            raise Exception(f"No step with id: {step_id}")
 
-        return d["step"]
+        return d.step
 
     def get_step_input_refs(self, step_id: str) -> Mapping[str, StepInputRef]:
 
         d = self.steps_details.get(step_id, None)
         if d is None:
-            raise Exception(f"No module with id: {step_id}")
+            raise Exception(f"No step with id: {step_id}")
 
-        return d["inputs"]
+        return d.inputs
 
     def get_step_output_refs(self, step_id: str) -> Mapping[str, StepOutputRef]:
 
         d = self.steps_details.get(step_id, None)
         if d is None:
-            raise Exception(f"No module with id: {step_id}")
+            raise Exception(f"No step with id: {step_id}")
 
-        return d["outputs"]
+        return d.outputs
 
-    def get_step_details(self, step_id: str) -> Mapping[str, Any]:
+    def get_step_details(self, step_id: str) -> StepInfo:
 
         d = self.steps_details.get(step_id, None)
         if d is None:
-            raise Exception(f"No module with id: {step_id}")
+            raise Exception(f"No step with id: {step_id}")
 
         return d
 
@@ -271,6 +306,14 @@ class PipelineStructure(KiaraModel):
             self._process_steps()
         return self._processing_stages
 
+    @property
+    def processing_stages_info(self) -> Mapping[int, PipelineStage]:
+        if self._stages_info is not None:
+            return self._stages_info
+
+        self._stages_info = PipelineStage.from_pipeline_structure(self)
+        return self._stages_info
+
     @lru_cache()
     def _get_node_of_type(self, node_type: str):
         if self._steps_details is None:
@@ -341,7 +384,7 @@ class PipelineStructure(KiaraModel):
     def step_is_required(self, step_id: str) -> bool:
         """Check if the specified step is required, or can be omitted."""
 
-        return self.get_step_details(step_id=step_id)["required"]
+        return self.get_step_details(step_id=step_id).required
 
     def _process_steps(self):
         """The core method of this class, it connects all the processing modules, their inputs and outputs."""
@@ -498,14 +541,19 @@ class PipelineStructure(KiaraModel):
                     # pipeline_input_name = generate_pipeline_endpoint_name(
                     #     step_id=step.step_id, value_name=input_name
                     # )
-                    pipeline_input_name = f"{step.step_id}.{input_name}"
+                    pipeline_input_ref = f"{step.step_id}.{input_name}"
+
                     # check whether this input has an alias associated with it
                     if not self.input_aliases:
                         raise NotImplementedError()
 
-                    if pipeline_input_name in self.input_aliases.keys():
+                    if pipeline_input_ref in self.input_aliases.keys():
                         # this means we use the pipeline alias
-                        pipeline_input_name = self.input_aliases[pipeline_input_name]
+                        pipeline_input_name = self.input_aliases[pipeline_input_ref]
+                    else:
+                        pipeline_input_name = generate_pipeline_endpoint_name(
+                            step_id=step.step_id, value_name=input_name
+                        )
 
                     if pipeline_input_name in existing_pipeline_input_points.keys():
                         # we already created a pipeline input with this name
@@ -553,6 +601,7 @@ class PipelineStructure(KiaraModel):
                         value_schema=schema,
                         connected_pipeline_input=connected_pipeline_input.value_name,
                         connected_outputs=None,
+                        is_constant=is_constant,
                     )
                     connected_pipeline_input.connected_inputs.append(
                         step_input_point.address
@@ -580,24 +629,63 @@ class PipelineStructure(KiaraModel):
             step_id = step.step_id
 
             paths = list(nx.all_simple_paths(execution_graph, "__root__", step_id))
+
             max_steps = max(paths, key=lambda x: len(x))
             path_lengths[step_id] = len(max_steps) - 1
 
-        max_length = max(path_lengths.values())
+        if path_lengths.values():
+            max_length = max(path_lengths.values())
 
-        for i in range(1, max_length + 1):
-            stage: List[str] = [m for m, length in path_lengths.items() if length == i]
-            processing_stages.append(stage)
-            for _step_id in stage:
-                steps_details[_step_id]["processing_stage"] = i
-                # steps_details[_step_id]["step"].processing_stage = i
+            for i in range(1, max_length + 1):
+                stage: List[str] = [
+                    m for m, length in path_lengths.items() if length == i
+                ]
+                processing_stages.append(stage)
+                for _step_id in stage:
+                    steps_details[_step_id]["processing_stage"] = i
+                    # steps_details[_step_id]["step"].processing_stage = i
 
         self._constants = constants
         self._defaults = structure_defaults
-        self._steps_details = steps_details
+        self._steps_details = {
+            step_id: StepInfo(**data) for step_id, data in steps_details.items()
+        }
         self._execution_graph = execution_graph
         self._data_flow_graph = data_flow_graph
         self._data_flow_graph_simple = data_flow_graph_simple
         self._processing_stages = processing_stages
 
         self._get_node_of_type.cache_clear()
+
+    # def info(self) -> "PipelineStructureInfo":
+    #
+    #     if self._info is not None:
+    #         return self._info
+    #
+    #     from kiara.interfaces.python_api.models.info import PipelineStructureInfo
+    #
+    #     self._info = PipelineStructureInfo.create_from_instance(kiara=None, instance=self)  # type: ignore
+    #     return self._info
+
+    def create_renderable(self, **config: Any) -> RenderableType:
+
+        tree = Tree("pipeline")
+        inputs = tree.add("inputs")
+        for field_name, schema in self.pipeline_inputs_schema.items():
+            inputs.add(f"[i]{field_name}[i] (type: {schema.type})")
+
+        steps = tree.add("steps")
+        for idx, stage in enumerate(self.processing_stages, start=1):
+            stage_node = steps.add(f"stage {idx}")
+            for step_id in stage:
+                step_node = stage_node.add(f"step: {step_id}")
+                step = self.get_step(step_id=step_id)
+                if step.doc.is_set:
+                    step_node.add(f"desc: {step.doc.description}")
+                step_node.add(f"module: {step.manifest_src.module_type}")
+
+        outputs = tree.add("outputs")
+        for field_name, schema in self.pipeline_outputs_schema.items():
+            outputs.add(f"[i]{field_name}[i] (type: {schema.type})")
+
+        return tree
