@@ -21,6 +21,7 @@ from pydantic import BaseModel, Extra, PrivateAttr, root_validator
 from pydantic.fields import Field
 from rich import box
 from rich.console import Group, RenderableType
+from rich.panel import Panel
 from rich.rule import Rule
 from rich.syntax import Syntax
 from rich.table import Table
@@ -33,8 +34,8 @@ from typing import (
     Literal,
     Mapping,
     MutableMapping,
-    Optional,
     Sequence,
+    Set,
     Union,
 )
 
@@ -48,10 +49,12 @@ from kiara.exceptions import InvalidValuesException
 from kiara.models import KiaraModel
 from kiara.models.module.manifest import InputsManifest, Manifest
 from kiara.models.python_class import PythonClass
-from kiara.models.values import ValueStatus
+from kiara.models.values import DataTypeCharacteristics, ValueStatus
 from kiara.models.values.value_schema import ValueSchema
-from kiara.utils import StringYAML, is_debug, orjson_dumps
+from kiara.utils import is_jupyter, log_exception
 from kiara.utils.hashing import create_cid_digest
+from kiara.utils.json import orjson_dumps
+from kiara.utils.yaml import StringYAML
 
 log = logging.getLogger("kiara")
 yaml = StringYAML()
@@ -59,6 +62,8 @@ yaml = StringYAML()
 if TYPE_CHECKING:
     from kiara.context import Kiara
     from kiara.data_types import DataType
+    from kiara.interfaces.python_api.models.info import ValueInfo
+    from kiara.models.values.lineage import ValueLineage
     from kiara.registries.data import DataRegistry
 
 
@@ -68,7 +73,7 @@ class SerializedChunks(BaseModel, abc.ABC):
         json_dumps = orjson_dumps
         extra = Extra.forbid
 
-    _size_cache: Optional[int] = PrivateAttr(default=None)
+    _size_cache: Union[int, None] = PrivateAttr(default=None)
     _hashes_cache: Dict[str, Sequence[CID]] = PrivateAttr(default_factory=dict)
 
     @abc.abstractmethod
@@ -108,7 +113,7 @@ class SerializedChunks(BaseModel, abc.ABC):
         return self._hashes_cache[hash_codec]
 
     def _store_bytes_to_file(
-        self, chunks: Iterable[bytes], file: Optional[str] = None
+        self, chunks: Iterable[bytes], file: Union[str, None] = None
     ) -> str:
         "Utility method to store bytes to a file."
 
@@ -138,40 +143,6 @@ class SerializedChunks(BaseModel, abc.ABC):
 
         return content
 
-    # @property
-    # def data_hashes(self) -> Iterable[bytes]:
-    #
-    #     if self._hash_cache is not None:
-    #         return self._hash_cache
-    #
-    #     result = []
-    #     size = 0
-    #     for chunk in self.get_chunks():
-    #         _hash = multihash.digest(chunk, self.codec)
-    #         size = size + len(chunk)
-    #         result.append(_hash)
-    #
-    #     if self._size_cache is None:
-    #         self._size_cache = size
-    #     else:
-    #         assert self._size_cache == size
-    #
-    #     self._hash_cache = result
-    #     return self._hash_cache
-
-    # @property
-    # def data_size(self) -> int:
-    #
-    #     if self._size_cache is not None:
-    #         return self._size_cache
-    #
-    #     size = 0
-    #     for chunk in self.get_chunks():
-    #         size = size + len(chunk)
-    #
-    #     self._size_cache = size
-    #     return self._size_cache
-
 
 class SerializedPreStoreChunks(SerializedChunks):
 
@@ -350,7 +321,7 @@ class SerializedInlineJson(SerializedPreStoreChunks):
     inline_data: Any = Field(
         description="Data that will not be stored externally, but inline in the containing model. This should only contain data types that can be serialized reliably using json (scalars, etc.)."
     )
-    _json_cache: Optional[bytes] = PrivateAttr(default=None)
+    _json_cache: Union[bytes, None] = PrivateAttr(default=None)
 
     def as_json(self) -> bytes:
         assert self.inline_data is not None
@@ -386,7 +357,7 @@ class SerializedChunkIDs(SerializedChunks):
     chunk_id_list: List[str] = Field(
         description="A list of chunk ids, which will be resolved via the attached data registry."
     )
-    archive_id: Optional[uuid.UUID] = Field(
+    archive_id: Union[uuid.UUID, None] = Field(
         description="The preferred data archive to get the chunks from."
     )
     size: int = Field(description="The size of all chunks combined.")
@@ -473,7 +444,7 @@ class SerializedData(KiaraModel):
     )
     metadata: SerializationMetadata = Field(
         description="Optional metadata describing aspects of the serialization used.",
-        default_factory=dict,
+        default_factory=SerializationMetadata,
     )
 
     hash_codec: str = Field(
@@ -481,8 +452,8 @@ class SerializedData(KiaraModel):
     )
     _cids_cache: Dict[str, Sequence[CID]] = PrivateAttr(default_factory=dict)
 
-    _cached_data_size: Optional[int] = PrivateAttr(default=None)
-    _cached_dag: Optional[Dict[str, Sequence[CID]]] = PrivateAttr(default=None)
+    _cached_data_size: Union[int, None] = PrivateAttr(default=None)
+    _cached_dag: Union[Dict[str, Sequence[CID]], None] = PrivateAttr(default=None)
     # _cached_cid: Optional[CID] = PrivateAttr(default=None)
 
     def _retrieve_data_to_hash(self) -> Any:
@@ -509,22 +480,6 @@ class SerializedData(KiaraModel):
     def get_serialized_data(self, key: str) -> SerializedChunks:
         pass
 
-    # @property
-    # def cid(self) -> CID:
-    #
-    #     if self._cached_cid is not None:
-    #         return self._cached_cid
-    #
-    #     # TODO: check whether that is correect, or whether it needs another wrapping in an 'identity' type
-    #     codec = multicodec.get("dag-cbor")
-    #
-    #     hash_func = Multihash(codec=self.hash_codec).digest
-    #     hash = hash_func(self.dag)
-    #     cid = create_cid_digest(codec, hash)
-    #     self._cached_cid = cid
-    #
-    #     return self._cached_cid
-
     def get_cids_for_key(self, key) -> Sequence[CID]:
 
         if key in self._cids_cache.keys():
@@ -654,26 +609,6 @@ class PersistedData(SerializedData):
         return self.chunk_id_map[key]
 
 
-# class LoadConfig(Manifest):
-#
-#     inputs: Mapping[str, PersistedData] = Field(
-#         description="A map translating from input field name to alias (key) in bytes_structure."
-#     )
-#     output_name: str = Field(
-#         description="The name of the field that contains the persisted value details."
-#     )
-#
-#     def _retrieve_data_to_hash(self) -> Any:
-#         return self.dict()
-#
-#     def __repr__(self):
-#
-#         return f"{self.__class__.__name__}(module_type={self.module_type}, output_name={self.output_name})"
-#
-#     def __str__(self):
-#         return self.__repr__()
-
-
 class ValuePedigree(InputsManifest):
 
     _kiara_model_id = "instance.value_pedigree"
@@ -699,6 +634,35 @@ class ValuePedigree(InputsManifest):
         return self.__repr__()
 
 
+class DataTypeInfo(KiaraModel):
+
+    _kiara_model_id = "info.data_type_instance"
+
+    data_type_name: str = Field(description="The registered name of this data type.")
+    data_type_config: Mapping[str, Any] = Field(
+        description="The (optional) configuration for this data type.",
+        default_factory=dict,
+    )
+    characteristics: DataTypeCharacteristics = Field(
+        description="Characteristics of this data type."
+    )
+    data_type_class: PythonClass = Field(
+        description="The python class that is associated with this model."
+    )
+    _data_type_instance: "DataType" = PrivateAttr(default=None)
+
+    @property
+    def data_type_instance(self) -> "DataType":
+
+        if self._data_type_instance is not None:
+            return self._data_type_instance
+
+        self._data_type_instance = self.data_type_class.get_class()(
+            **self.data_type_config
+        )
+        return self._data_type_instance
+
+
 class ValueDetails(KiaraModel):
     """A wrapper class that manages and retieves value data and its details."""
 
@@ -723,8 +687,8 @@ class ValueDetails(KiaraModel):
     pedigree_output_name: str = Field(
         description="The output name that produced this value (using the manifest inside the pedigree)."
     )
-    data_type_class: PythonClass = Field(
-        description="The python class that is associtated with this model."
+    data_type_info: DataTypeInfo = Field(
+        description="Information about the data type this value is made of."
     )
 
     def _retrieve_id(self) -> str:
@@ -739,11 +703,11 @@ class ValueDetails(KiaraModel):
 
     @property
     def data_type_name(self) -> str:
-        return self.value_schema.type
+        return self.data_type_info.data_type_name
 
     @property
     def data_type_config(self) -> Mapping[str, Any]:
-        return self.value_schema.type_config
+        return self.data_type_info.data_type_config
 
     @property
     def is_optional(self) -> bool:
@@ -800,14 +764,15 @@ class Value(ValueDetails):
     _serialized_data: Union[None, str, SerializedData] = PrivateAttr(default=None)
     _data_retrieved: bool = PrivateAttr(default=False)
     _data_registry: "DataRegistry" = PrivateAttr(default=None)
-    _data_type: "DataType" = PrivateAttr(default=None)
+    # _data_type: "DataType" = PrivateAttr(default=None)
     _is_stored: bool = PrivateAttr(default=False)
-    _cached_properties: Optional["ValueMap"] = PrivateAttr(default=None)
+    _cached_properties: Union["ValueMap", None] = PrivateAttr(default=None)
+    _lineage: Union["ValueLineage", None] = PrivateAttr(default=None)
 
     environment_hashes: Mapping[str, Mapping[str, str]] = Field(
         description="Hashes for the environments this value was created in."
     )
-    enviroments: Optional[Mapping[str, Mapping[str, Any]]] = Field(
+    enviroments: Union[Mapping[str, Mapping[str, Any]], None] = Field(
         description="Information about the environments this value was created in.",
         default=None,
     )
@@ -840,7 +805,7 @@ class Value(ValueDetails):
 
         if add_origin_to_property_value:
             if value is None:
-                value = self._data_registry.get_value(value_id=value_id)  # type: ignore
+                value = self._data_registry.get_value(value=value_id)  # type: ignore
 
             if value._is_stored:
                 raise Exception(
@@ -890,6 +855,11 @@ class Value(ValueDetails):
 
         return False
 
+    # @property
+    # def data_type_class(self) -> "PythonClass":
+    #     """Return the (Python) type of the underlying 'DataType' subclass."""
+    #     return self.data_type_info.data_type_class
+
     @property
     def serialized_data(self) -> SerializedData:
 
@@ -963,12 +933,17 @@ class Value(ValueDetails):
     @property
     def data_type(self) -> "DataType":
 
-        if self._data_type is not None:
-            return self._data_type
+        return self.data_type_info.data_type_instance
 
-        cls = self.data_type_class.get_class()
-        self._data_type = cls(**self.value_schema.type_config)
-        return self._data_type
+    @property
+    def lineage(self) -> "ValueLineage":
+        if self._lineage is not None:
+            return self._lineage
+
+        from kiara.models.values.lineage import ValueLineage
+
+        self._lineage = ValueLineage(kiara=self._data_registry._kiara, value=self)
+        return self._lineage
 
     @property
     def property_values(self) -> "ValueMap":
@@ -996,6 +971,145 @@ class Value(ValueDetails):
 
         return self.get_property_value(property_key=property_key).data
 
+    def get_all_property_data(self) -> Mapping[str, Any]:
+
+        return {k: self.get_property_data(k) for k in self.property_names}
+
+    def lookup_self_aliases(self) -> Set[str]:
+
+        if not self._data_registry:
+            raise Exception(
+                f"Can't lookup aliases for value '{self.value_id}': data registry not set (yet)."
+            )
+
+        return self._data_registry.lookup_aliases(self)
+
+    def create_info(self) -> "ValueInfo":
+
+        if not self._data_registry:
+            raise Exception(
+                f"Can't create info object for value '{self.value_id}': data registry not set (yet)."
+            )
+
+        return self._data_registry.create_value_info(value=self.value_id)
+
+    def create_info_data(self, **config: Any) -> Mapping[str, Any]:
+
+        show_pedigree = config.get("show_pedigree", False)
+        show_lineage = config.get("show_lineage", False)
+        show_properties = config.get("show_properties", False)
+        # show_destinies = config.get("show_destinies", False)
+        # show_destiny_backlinks = config.get("show_destiny_backlinks", False)
+        # show_data = config.get("show_data_preview", False)
+        show_serialized = config.get("show_serialized", False)
+        show_env_data_hashes = config.get("show_environment_hashes", False)
+        show_env_data = config.get("show_environment_data", False)
+
+        ignore_fields = config.get("ignore_fields", [])
+
+        table: Dict[str, Any] = {}
+
+        if "value_id" not in ignore_fields:
+            table["value_id"] = self.value_id
+        if "aliases" not in ignore_fields:
+            if hasattr(self, "aliases"):
+                table["aliases"] = self.aliases  # type: ignore
+
+        if "kiara_id" not in ignore_fields:
+            table["kiara_id"] = self.kiara_id
+
+        for k in sorted(self.__fields__.keys()):
+
+            if (
+                k
+                in [
+                    "serialized",
+                    "value_id",
+                    "aliases",
+                    "kiara_id",
+                    "environments",
+                    "environment_hashes",
+                ]
+                or k in ignore_fields
+            ):
+                continue
+
+            attr = getattr(self, k)
+            if k in ["pedigree_output_name", "pedigree"]:
+                continue
+            else:
+                v = attr
+
+            table[k] = v
+
+        if show_pedigree:
+            pedigree = getattr(self, "pedigree")
+
+            table["pedigree"] = pedigree
+            if pedigree == ORPHAN:
+                pedigree_output_name: Union[Any, None] = None
+            else:
+                pedigree_output_name = getattr(self, "pedigree_output_name")
+
+            table["pedigree_output_name"] = pedigree_output_name
+
+        if show_lineage:
+            table["lineage"] = self.lineage
+
+        if show_serialized:
+            serialized = self._data_registry.retrieve_persisted_value_details(
+                self.value_id
+            )
+            table["serialized"] = serialized
+
+        if show_env_data_hashes:
+            env_hashes = Syntax(
+                orjson_dumps(self.environment_hashes, option=orjson.OPT_INDENT_2),
+                "json",
+                background_color="default",
+            )
+            table["environment_hashes"] = env_hashes
+
+        if show_env_data:
+            raise NotImplementedError()
+
+        if show_properties:
+            if not self.property_links:
+                table["properties"] = {}
+            else:
+                properties = self._data_registry.load_values(self.property_links)
+                table["properties"] = properties
+
+        # if hasattr(self, "destiny_links") and show_destinies:
+        #     if not self.destiny_links:  # type: ignore
+        #         table["destinies"] = {}
+        #     else:
+        #         destinies = self._data_registry.load_values(self.destiny_links)  # type: ignore
+        #         table["destinies"] = destinies
+        #
+        # if show_destiny_backlinks:
+        #     if not self.destiny_backlinks:
+        #         table["destiny backlinks"] = {}
+        #     else:
+        #         destiny_items: List[Any] = []
+        #         for v_id, alias in self.destiny_backlinks.items():
+        #             destiny_items.append(
+        #                 f"[b]Value: [i]{v_id}[/i] (destiny alias: {alias})[/b]"
+        #             )
+        #             rendered = self._data_registry.pretty_print_data(
+        #                 value_id=v_id, **config
+        #             )
+        #             destiny_items.append(rendered)
+        #         table["destiny backlinks"] = destiny_items
+        #
+        # if show_data:
+        #     rendered = self._data_registry.pretty_print_data(
+        #         self.value_id, target_type="terminal_renderable"
+        #     )
+        #     table["data preview"] = rendered
+
+        return table
+
     def create_renderable(self, **render_config: Any) -> RenderableType:
 
         from kiara.utils.output import extract_renderable
@@ -1010,38 +1124,51 @@ class Value(ValueDetails):
         show_env_data_hashes = render_config.get("show_environment_hashes", False)
         show_env_data = render_config.get("show_environment_data", False)
 
+        ignore_fields = render_config.get("ignore_fields", [])
+
         table = Table(show_header=False, box=box.SIMPLE)
         table.add_column("Key", style="i")
         table.add_column("Value")
 
-        table.add_row("value_id", str(self.value_id))
-        if hasattr(self, "aliases"):
-            if not self.aliases:  # type: ignore
-                aliases_str = "-- n/a --"
-            else:
-                aliases_str = ", ".join(self.aliases)  # type: ignore
-            table.add_row("aliases", aliases_str)
-        table.add_row("kiara_id", str(self.kiara_id))
+        info_data = self.create_info_data(**render_config)
+
+        if "value_id" not in ignore_fields:
+            table.add_row("value_id", str(info_data["value_id"]))
+        if "aliases" not in ignore_fields:
+            if info_data.get("aliases", None):
+                aliases_str = ", ".join(info_data["aliases"])  # type: ignore
+                table.add_row("aliases", aliases_str)
+            # else:
+            #     aliases_str = "-- n/a --"
+            #     table.add_row("aliases", aliases_str)
+
+        if "kiara_id" not in ignore_fields:
+            table.add_row("kiara_id", str(info_data["kiara_id"]))
+
         table.add_row("", "")
         table.add_row("", Rule())
-        for k in sorted(self.__fields__.keys()):
+        for k in sorted(info_data.keys()):
 
-            if k in [
-                "serialized",
-                "value_id",
-                "aliases",
-                "kiara_id",
-                "environments",
-                "environment_hashes",
-            ]:
+            if (
+                k
+                in [
+                    "serialized",
+                    "value_id",
+                    "aliases",
+                    "kiara_id",
+                    "environments",
+                    "environment_hashes",
+                ]
+                or k in ignore_fields
+            ):
                 continue
 
-            attr = getattr(self, k)
+            attr = info_data[k]
             if k in ["pedigree_output_name", "pedigree"]:
                 continue
 
             elif k == "value_status":
-                v = f"[i]-- {attr.value} --[/i]"
+                v: RenderableType = f"[i]-- {attr.value} --[/i]"
             elif k == "value_size":
                 v = format_size(attr)
             else:
@@ -1064,14 +1191,14 @@ class Value(ValueDetails):
             table.add_row("", "")
 
         if show_pedigree:
-            pedigree = getattr(self, "pedigree")
+            pedigree = info_data["pedigree"]
 
             if pedigree == ORPHAN:
                 v = "[i]-- external data --[/i]"
-                pedigree_output_name: Optional[Any] = None
+                pedigree_output_name: Union[Any, None] = None
             else:
                 v = extract_renderable(pedigree)
-                pedigree_output_name = getattr(self, "pedigree_output_name")
+                pedigree_output_name = info_data["pedigree_output_name"]
 
             row = ["pedigree", v]
             table.add_row(*row)
@@ -1080,20 +1207,19 @@ class Value(ValueDetails):
                 table.add_row(*row)
 
         if show_lineage:
-            from kiara.models.values.lineage import ValueLineage
-
-            vl = ValueLineage(kiara=self._data_registry._kiara, value=self)
-            table.add_row("lineage", vl.create_renderable(include_ids=True))
+            table.add_row(
+                "lineage", info_data["lineage"].create_renderable(include_ids=True)
+            )
 
         if show_serialized:
-            serialized = self._data_registry.retrieve_persisted_value_details(
-                self.value_id
-            )
+            serialized = info_data["serialized"]
             table.add_row("serialized", serialized.create_renderable())
 
         if show_env_data_hashes:
             env_hashes = Syntax(
-                orjson_dumps(self.environment_hashes, option=orjson.OPT_INDENT_2),
+                orjson_dumps(
+                    info_data["environment_hashes"], option=orjson.OPT_INDENT_2
+                ),
                 "json",
                 background_color="default",
             )
@@ -1103,10 +1229,10 @@ class Value(ValueDetails):
             raise NotImplementedError()
 
         if show_properties:
-            if not self.property_links:
+            if not info_data["properties"]:
                 table.add_row("properties", "{}")
             else:
-                properties = self._data_registry.load_values(self.property_links)
+                properties = info_data["properties"]
                 pr = properties.create_renderable(show_header=False)
                 table.add_row("properties", pr)
 
@@ -1196,7 +1322,7 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
             item = self.get_value_obj(field_name)
             field_schema = self.values_schema[field_name]
             if not field_schema.optional:
-                msg: Optional[str] = None
+                msg: Union[str, None] = None
                 if not item.value_status == ValueStatus.SET:
 
                     item_schema = self.values_schema[field_name]
@@ -1294,7 +1420,7 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
     def __str__(self):
         return self.__repr__()
 
-    def create_invalid_renderable(self, **config) -> Optional[RenderableType]:
+    def create_invalid_renderable(self, **config) -> Union[RenderableType, None]:
 
         inv = self.check_invalid()
         if not inv:
@@ -1311,6 +1437,13 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
 
     def create_renderable(self, **config: Any) -> RenderableType:
 
+        in_panel = config.get("in_panel", None)
+        if in_panel is None:
+            if is_jupyter():
+                in_panel = True
+            else:
+                in_panel = False
+
         render_value_data = config.get("render_value_data", True)
         field_title = config.get("field_title", "field")
         value_title = config.get("value_title", "value")
@@ -1338,7 +1471,10 @@ class ValueMap(KiaraModel, MutableMapping[str, Value]):  # type: ignore
             else:
                 table.add_row(field_name, rendered)
 
-        return table
+        if in_panel:
+            return Panel(table)
+        else:
+            return table
 
 
 class ValueMapReadOnly(ValueMap):  # type: ignore
@@ -1381,7 +1517,9 @@ class ValueMapWritable(ValueMap):  # type: ignore
     ) -> "ValueMapWritable":
 
         v = ValueMapWritable(
-            values_schema=schema, pedigree=pedigree, unique_value_ids=unique_value_ids
+            values_schema=dict(schema),
+            pedigree=pedigree,
+            unique_value_ids=unique_value_ids,
         )
         v._kiara = kiara
         v._data_registry = kiara.data_registry
@@ -1395,7 +1533,7 @@ class ValueMapWritable(ValueMap):  # type: ignore
     )
     unique_value_ids: bool = Field(
         description="Whether this value map always creates new value(id)s, even when a dataset with matching hash is found.",
-        default=False,
+        default=True,
     )
 
     _values_uncommitted: Dict[str, Any] = PrivateAttr(default_factory=dict)
@@ -1448,11 +1586,13 @@ class ValueMapWritable(ValueMap):  # type: ignore
 
         invalid = self.check_invalid()
         if invalid:
-            if is_debug():
-                import traceback
-
-                traceback.print_stack()
-            raise InvalidValuesException(invalid_values=invalid)
+            e = InvalidValuesException(invalid_values=invalid)
+            try:
+                raise e
+            except Exception:
+                # this is silly, I know
+                log_exception(e)
+                raise e
 
     def set_value(self, field_name: str, data: Any) -> None:
         """Set the value for the specified field."""
