@@ -7,6 +7,7 @@
 
 import abc
 import inspect
+import structlog
 import uuid
 from abc import abstractmethod
 from multiformats import CID
@@ -19,7 +20,6 @@ from typing import (
     Generic,
     Iterable,
     Mapping,
-    Optional,
     Type,
     TypeVar,
     Union,
@@ -29,15 +29,17 @@ from kiara.exceptions import KiaraModuleConfigException
 from kiara.models.module import KiaraModuleConfig
 from kiara.models.module.jobs import JobLog
 from kiara.models.values.value_schema import ValueSchema
-from kiara.utils import StringYAML, is_debug
+from kiara.utils import is_debug, log_exception
 from kiara.utils.hashing import compute_cid
 from kiara.utils.values import (
     augment_values,
     create_schema_dict,
     overlay_constants_and_defaults,
 )
+from kiara.utils.yaml import StringYAML
 
 if TYPE_CHECKING:
+    from kiara.models.module.manifest import Manifest
     from kiara.models.values.value import ValueMap
     from kiara.operations import Operation
 
@@ -45,7 +47,12 @@ yaml = StringYAML()
 
 KIARA_CONFIG = TypeVar("KIARA_CONFIG", bound=KiaraModuleConfig)
 
-ValueSetSchema = Mapping[str, Union[ValueSchema, Mapping[str, Any]]]
+ValueMapSchema = Mapping[str, Union[ValueSchema, Mapping[str, Any]]]
+ValueSetSchema = Mapping[
+    str, Union[ValueSchema, Mapping[str, Any]]
+]  # TODO: remove once all references to this are gone (legacy)
+
+log = structlog.getLogger()
 
 
 class InputOutputObject(abc.ABC):
@@ -111,13 +118,13 @@ class InputOutputObject(abc.ABC):
     @abstractmethod
     def create_inputs_schema(
         self,
-    ) -> ValueSetSchema:
+    ) -> ValueMapSchema:
         """Return the schema for this types' inputs."""
 
     @abstractmethod
     def create_outputs_schema(
         self,
-    ) -> ValueSetSchema:
+    ) -> ValueMapSchema:
         """Return the schema for this types' outputs."""
 
     @property
@@ -147,7 +154,11 @@ class InputOutputObject(abc.ABC):
         return self._constants  # type: ignore
 
     def _create_inputs_schema(self) -> None:
+        """Assemble the inputs schema and assign it to the approriate instance attributes.
 
+        DEV NOTE: if anything in this method is changed, also change the method of the AutoInputsKiaraModule
+        in the kiara_pluginc.core_types package, since it's a carbon copy if this, except for a small change.
+        """
         try:
             _input_schemas_data = self.create_inputs_schema()
 
@@ -248,7 +259,7 @@ class ModuleCharacteristics(BaseModel):
 
     is_idempotent: bool = Field(
         description="Whether this module is idempotent (aka always produces the same output with the same inputs.",
-        default=False,
+        default=True,
     )
     is_internal: bool = Field(
         description="Hint for frontends whether this module is used predominantly internally, and users won't need to know of its existence.",
@@ -256,11 +267,17 @@ class ModuleCharacteristics(BaseModel):
     )
     unique_result_values: bool = Field(
         description="Don't re-use existing values for outputs that have matching hashes in the data store.",
-        default=False,
+        default=True,
     )
 
 
 DEFAULT_IDEMPOTENT_MODULE_CHARACTERISTICS = ModuleCharacteristics()
+DEFAULT_NO_IDEMPOTENT_MODULE_CHARACTERISTICS = ModuleCharacteristics(
+    is_idempotent=False
+)
+DEFAULT_IDEMPOTENT_INTERNAL_MODULE_CHARACTERISTICS = ModuleCharacteristics(
+    is_internal=True
+)
 
 
 class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
@@ -305,15 +322,22 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
     ) -> CID:
 
         if isinstance(module_type_config, Mapping):
-            module_type_config = cls._config_cls(**module_type_config)
+            module_type_config = cls._resolve_module_config(**module_type_config)
 
         obj = {
             "module_type": cls._module_type_name,  # type: ignore
-            "module_type_config": module_type_config.dict(),  # type: ignore
+            "module_type_config": module_type_config.dict(),
         }
         _, cid = compute_cid(data=obj)
         return cid
 
+    @classmethod
+    def _resolve_module_config(cls, **config: Any) -> KIARA_CONFIG:
+
+        _config = cls._config_cls(**config)  # type: ignore
+
+        return _config
+
     def __init__(
         self,
         module_config: Union[None, KIARA_CONFIG, Mapping[str, Any]] = None,
@@ -337,13 +361,24 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
         else:
             raise TypeError(f"Invalid type for module config: {type(module_config)}")
 
-        self._module_cid: Optional[CID] = None
-        self._characteristics: Optional[ModuleCharacteristics] = None
+        self._module_cid: Union[CID, None] = None
+        self._characteristics: Union[ModuleCharacteristics, None] = None
 
         super().__init__(alias=self.__class__._module_type_name, config=self._config)  # type: ignore
 
-        self._operation: Optional[Operation] = None
+        self._operation: Union[Operation, None] = None
         # self._merged_input_schemas: typing.Mapping[str, ValueSchema] = None  # type: ignore
+        self._manifest_cache: Union[None, Manifest] = None
+
+    @property
+    def manifest(self) -> "Manifest":
+        if self._manifest_cache is None:
+            self._manifest_cache = Manifest(
+                module_type=self.module_type_name,
+                module_config=self.config.dict(),
+                is_resolved=True,
+            )
+        return self._manifest_cache
 
     @property
     def module_id(self) -> uuid.UUID:
@@ -418,33 +453,19 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
 
         signature = inspect.signature(self.process)  # type: ignore
 
-        if "job_log" not in signature.parameters.keys():
+        process_inputs: Dict[str, Any] = {
+            "inputs": inputs,
+            "outputs": outputs,
+        }
 
-            try:
-                self.process(inputs=inputs, outputs=outputs)  # type: ignore
-            except Exception as e:
-                if is_debug():
-                    try:
-                        import traceback
+        if "job_log" in signature.parameters.keys():
+            process_inputs["job_log"] = job_log
 
-                        traceback.print_exc()
-                    except Exception:
-                        pass
-                raise e
-
-        else:
-
-            try:
-                self.process(inputs=inputs, outputs=outputs, job_log=job_log)  # type: ignore
-            except Exception as e:
-                if is_debug():
-                    try:
-                        import traceback
-
-                        traceback.print_exc()
-                    except Exception:
-                        pass
-                raise e
+        try:
+            self.process(**process_inputs)  # type: ignore
+        except Exception as e:
+            log_exception(e)
+            raise e
 
     def __eq__(self, other):
         if self.__class__ != other.__class__:
@@ -457,7 +478,8 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
     def __repr__(self):
         return f"{self.__class__.__name__}(id={self.module_id} module_type={self.module_type_name} input_names={list(self.input_names)} output_names={list(self.output_names)})"
 
-    def create_renderable(self, **config) -> RenderableType:
+    @property
+    def operation(self) -> "Operation":
 
         if self._operation is not None:
             return self._operation
@@ -466,3 +488,7 @@ class KiaraModule(InputOutputObject, Generic[KIARA_CONFIG]):
 
         self._operation = Operation.create_from_module(self)
         return self._operation
+
+    def create_renderable(self, **config) -> RenderableType:
+
+        return self.operation.create_renderable(**config)
