@@ -5,17 +5,17 @@
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
-import orjson
-import pprint
-from pydantic import BaseModel, Field, validator
-from typing import Any, Dict, Iterable, Mapping, Tuple, Union
+from pydantic import Field, validator
+from typing import Any, Iterable, Mapping, Tuple, Union
 
-from kiara.models import KiaraModel
 from kiara.models.module import KiaraModuleConfig
-from kiara.models.values.value import Value, ValueMap
+from kiara.models.values.value import ValueMap
 from kiara.models.values.value_schema import ValueSchema
-from kiara.modules import KiaraModule
-from kiara.utils.output import create_table_from_model_object
+from kiara.modules import (
+    DEFAULT_IDEMPOTENT_INTERNAL_MODULE_CHARACTERISTICS,
+    KiaraModule,
+    ModuleCharacteristics,
+)
 
 
 class PrettyPrintConfig(KiaraModuleConfig):
@@ -57,6 +57,9 @@ class PrettyPrintModule(KiaraModule):
     # def create_persistence_config_schema(self) -> Optional[Mapping[str, Mapping[str, Any]]]:
     #     return None
 
+    def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
+        return DEFAULT_IDEMPOTENT_INTERNAL_MODULE_CHARACTERISTICS
+
     def create_inputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
@@ -65,7 +68,7 @@ class PrettyPrintModule(KiaraModule):
         assert source_type not in ["target", "base_name"]
 
         schema = {
-            source_type: {"type": source_type, "doc": "The value to render."},
+            "value": {"type": source_type, "doc": "The value to render."},
             "render_config": {
                 "type": "any",
                 "doc": "Value type dependent render configuration.",
@@ -91,13 +94,17 @@ class PrettyPrintModule(KiaraModule):
         source_type = self.get_config_value("source_type")
         target_type = self.get_config_value("target_type")
 
-        value = inputs.get_value_obj(source_type)
+        value = inputs.get_value_obj("value")
         render_config = inputs.get_value_data("render_config")
 
         func_name = f"pretty_print__{source_type}__as__{target_type}"
 
         func = getattr(self, func_name)
         # TODO: check function signature is valid
+
+        if render_config is None:
+            render_config = {}
+
         result = func(value=value, render_config=render_config)
 
         outputs.set_value("rendered_value", result)
@@ -108,6 +115,9 @@ class ValueTypePrettyPrintModule(KiaraModule):
     _module_type_name = "pretty_print.value"
     _config_cls = PrettyPrintConfig
 
+    def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
+        return DEFAULT_IDEMPOTENT_INTERNAL_MODULE_CHARACTERISTICS
+
     def create_inputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
@@ -116,7 +126,7 @@ class ValueTypePrettyPrintModule(KiaraModule):
         assert source_type not in ["target", "base_name"]
 
         schema = {
-            source_type: {
+            "value": {
                 "type": source_type,
                 "doc": "The value to render.",
                 "optional": True,
@@ -143,17 +153,17 @@ class ValueTypePrettyPrintModule(KiaraModule):
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
 
-        source_type = self.get_config_value("source_type")
+        # source_type = self.get_config_value("source_type")
         target_type = self.get_config_value("target_type")
 
-        source_value = inputs.get_value_obj(source_type)
+        source_value = inputs.get_value_obj("value")
         render_config = inputs.get_value_obj("render_config")
 
         if not source_value.is_set:
             outputs.set_value("rendered_value", "-- none/not set --")
             return
 
-        data_type_cls = source_value.data_type_class.get_class()
+        data_type_cls = source_value.data_type_info.data_type_class.get_class()
         data_type = data_type_cls(**source_value.value_schema.type_config)
 
         func_name = f"pretty_print_as__{target_type}"
@@ -172,26 +182,10 @@ class PrettyPrintAnyValueModule(PrettyPrintModule):
 
     _module_type_name = "pretty_print.any.value"
 
-    def render__any__as__string(self, value: Value, render_config: Dict[str, Any]):
-
-        data = value.data
-        if isinstance(data, KiaraModel):
-            return data.json(option=orjson.OPT_INDENT_2)
-        else:
-            return str(data)
-
-    def render__any__as__terminal_renderable(
-        self, value: Value, render_config: Dict[str, Any]
-    ):
-
-        data = value.data
-
-        if isinstance(data, BaseModel):
-            rendered = create_table_from_model_object(
-                model=data, render_config=render_config
-            )
-        elif isinstance(data, Iterable):
-            rendered = pprint.pformat(data)
-        else:
-            rendered = str(data)
-        return rendered
+    # def pretty_print__any__as__string(self, value: Value, render_config: Dict[str, Any]):
+    #
+    #     data = value.data
+    #     if isinstance(data, KiaraModel):
+    #         return data.json(option=orjson.OPT_INDENT_2)
+    #     else:
+    #         return str(data)
