@@ -6,12 +6,13 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 import abc
-from typing import TYPE_CHECKING, Generic, Iterable, Mapping, Optional, TypeVar, Union
+from typing import TYPE_CHECKING, Generic, Iterable, Mapping, TypeVar, Union
 
 from kiara.models.module.operation import Operation, OperationConfig, OperationDetails
 
 if TYPE_CHECKING:
-    from kiara import Kiara, KiaraModule
+    from kiara.context import Kiara
+    from kiara.modules import KiaraModule
 
 
 OPERATION_TYPE_DETAILS = TypeVar("OPERATION_TYPE_DETAILS", bound=OperationDetails)
@@ -39,7 +40,7 @@ class OperationType(abc.ABC, Generic[OPERATION_TYPE_DETAILS]):
     @abc.abstractmethod
     def check_matching_operation(
         self, module: "KiaraModule"
-    ) -> Optional[OPERATION_TYPE_DETAILS]:
+    ) -> Union[OPERATION_TYPE_DETAILS, None]:
         """Check whether the provided module is a valid operation for this type."""
 
     def retrieve_operation_details(
