@@ -4,6 +4,7 @@
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
+import abc
 import copy
 import structlog
 import uuid
@@ -15,7 +16,7 @@ from typing import (
     Dict,
     List,
     Mapping,
-    Optional,
+    Protocol,
     Set,
     Tuple,
     Union,
@@ -39,6 +40,7 @@ from kiara.exceptions import (
     NoSuchValueException,
     NoSuchValueIdException,
 )
+from kiara.interfaces.python_api.models.info import ValueInfo
 from kiara.models.events.data_registry import (
     DataArchiveAddedEvent,
     ValueCreatedEvent,
@@ -48,9 +50,11 @@ from kiara.models.events.data_registry import (
 )
 from kiara.models.module.operation import Operation
 from kiara.models.python_class import PythonClass
-from kiara.models.values import ValueStatus
+from kiara.models.values import DEFAULT_SCALAR_DATATYPE_CHARACTERISTICS, ValueStatus
+from kiara.models.values.matchers import ValueMatcher
 from kiara.models.values.value import (
     ORPHAN,
+    DataTypeInfo,
     PersistedData,
     SerializationMetadata,
     SerializedData,
@@ -72,6 +76,11 @@ if TYPE_CHECKING:
 logger = structlog.getLogger()
 
 
+class ValueLink(Protocol):
+
+    value_id: uuid.UUID
+
+
 NONE_PERSISTED_DATA = PersistedData.construct(
     data_type="none",
     data_type_config={},
@@ -83,6 +92,55 @@ NONE_PERSISTED_DATA = PersistedData.construct(
 )
 
 
+class AliasResolver(abc.ABC):
+    def __init__(self, kiara: "Kiara"):
+
+        self._kiara: "Kiara" = kiara
+
+    @abc.abstractmethod
+    def resolve_alias(self, alias: str) -> uuid.UUID:
+        pass
+
+
+class DefaultAliasResolver(AliasResolver):
+    def __init__(self, kiara: "Kiara"):
+
+        super().__init__(kiara=kiara)
+
+    def resolve_alias(self, alias: str) -> uuid.UUID:
+
+        if ":" in alias:
+            ref_type, rest = alias.split(":", maxsplit=1)
+
+            if ref_type == "value":
+                _value_id: Union[uuid.UUID, None] = uuid.UUID(rest)
+            elif ref_type == "alias":
+                _value_id = self._kiara.alias_registry.find_value_id_for_alias(
+                    alias=rest
+                )
+                if _value_id is None:
+                    raise NoSuchValueAliasException(
+                        alias=rest,
+                        msg=f"Can't retrive value for alias '{rest}': no such alias registered.",
+                    )
+            else:
+                raise Exception(
+                    f"Can't retrieve value for '{alias}': invalid reference type '{ref_type}'."
+                )
+        else:
+            _value_id = self._kiara.alias_registry.find_value_id_for_alias(alias)
+            if _value_id is None:
+                raise Exception(
+                    f"Can't retrieve value for alias '{alias}': no such alias registered."
+                )
+
+        if _value_id is None:
+            raise Exception(
+                f"Can't retrieve value for alias '{alias}': no such alias registered."
+            )
+        return _value_id
+
+
 class DataRegistry(object):
     def __init__(self, kiara: "Kiara"):
 
@@ -92,18 +150,26 @@ class DataRegistry(object):
 
         self._data_archives: Dict[str, DataArchive] = {}
 
-        self._default_data_store: Optional[str] = None
+        self._default_data_store: Union[str, None] = None
         self._registered_values: Dict[uuid.UUID, Value] = {}
 
         self._value_archive_lookup_map: Dict[uuid.UUID, str] = {}
+        """A cached dict that stores which archives which value ids belong to."""
 
         self._values_by_hash: Dict[str, Set[uuid.UUID]] = {}
 
         self._cached_data: Dict[uuid.UUID, Any] = {}
-        self._persisted_value_descs: Dict[uuid.UUID, Optional[PersistedData]] = {}
+        self._persisted_value_descs: Dict[uuid.UUID, Union[PersistedData, None]] = {}
+
+        self._alias_resolver: AliasResolver = DefaultAliasResolver(kiara=self._kiara)
 
         # initialize special values
         special_value_cls = PythonClass.from_class(NoneType)
+        data_type_info = DataTypeInfo.construct(
+            data_type_name="none",
+            characteristics=DEFAULT_SCALAR_DATATYPE_CHARACTERISTICS,
+            data_type_class=special_value_cls,
+        )
         self._not_set_value: Value = Value(
             value_id=NOT_SET_VALUE_ID,
             kiara_id=self._kiara.id,
@@ -119,7 +185,7 @@ class DataRegistry(object):
             value_hash=INVALID_HASH_MARKER,
             pedigree=ORPHAN,
             pedigree_output_name="__void__",
-            data_type_class=special_value_cls,
+            data_type_info=data_type_info,
         )
         self._not_set_value._data_registry = self
         self._cached_data[NOT_SET_VALUE_ID] = SpecialValue.NOT_SET
@@ -141,7 +207,7 @@ class DataRegistry(object):
             value_hash=str(NONE_CID),
             pedigree=ORPHAN,
             pedigree_output_name="__void__",
-            data_type_class=special_value_cls,
+            data_type_info=data_type_info,
         )
         self._none_value._data_registry = self
         self._cached_data[NONE_VALUE_ID] = SpecialValue.NO_VALUE
@@ -165,7 +231,8 @@ class DataRegistry(object):
         result: Set[uuid.UUID] = set()
         for store in self._data_archives.values():
             ids = store.value_ids
-            result.update(ids)
+            if ids:
+                result.update(ids)
 
         return result
 
@@ -173,7 +240,7 @@ class DataRegistry(object):
         self,
         archive: DataArchive,
         alias: str = None,
-        set_as_default_store: Optional[bool] = None,
+        set_as_default_store: Union[bool, None] = None,
     ):
 
         data_store_id = archive.archive_id
@@ -255,7 +322,7 @@ class DataRegistry(object):
             f"Can't retrieve archive with id '{archive_id}': no archive with that id registered."
         )
 
-    def find_store_id_for_value(self, value_id: uuid.UUID) -> Optional[str]:
+    def find_store_id_for_value(self, value_id: uuid.UUID) -> Union[str, None]:
 
         if value_id in self._value_archive_lookup_map.keys():
             return self._value_archive_lookup_map[value_id]
@@ -276,59 +343,40 @@ class DataRegistry(object):
         self._value_archive_lookup_map[value_id] = matches[0]
         return matches[0]
 
-    def get_value(self, value_id: Union[uuid.UUID, Value, str]) -> Value:
+    def get_value(self, value: Union[uuid.UUID, ValueLink, str]) -> Value:
         _value_id = None
 
-        if not isinstance(value_id, uuid.UUID):
+        if not isinstance(value, uuid.UUID):
             # fallbacks for common mistakes, this should error out if not a Value or string.
-            if hasattr(value_id, "value_id"):
-                _value_id: Optional[uuid.UUID] = value_id.value_id  # type: ignore
+            if hasattr(value, "value_id"):
+                _value_id: Union[uuid.UUID, str, None] = value.value_id  # type: ignore
+                if isinstance(_value_id, str):
+                    _value_id = uuid.UUID(_value_id)
             else:
 
                 try:
                     _value_id = uuid.UUID(
-                        value_id  # type: ignore
+                        value  # type: ignore
                     )  # this should fail if not string or wrong string format
                 except ValueError:
                     _value_id = None
 
                 if _value_id is None:
 
-                    if not isinstance(value_id, str):
+                    if not isinstance(value, str):
                         raise Exception(
-                            f"Can't retrieve value for '{value_id}': invalid type '{type(value_id)}'."
+                            f"Can't retrieve value for '{value}': invalid type '{type(value)}'."
                         )
 
-                    if ":" not in value_id:
-                        raise Exception(
-                            f"Can't retrieve value for '{value_id}': can't determine reference type."
-                        )
-
-                    ref_type, rest = value_id.split(":", maxsplit=1)
-
-                    if ref_type == "value":
-                        _value_id = uuid.UUID(rest)
-                    elif ref_type == "alias":
-                        _value_id = self._kiara.alias_registry.find_value_id_for_alias(
-                            alias=rest
-                        )
-                        if _value_id is None:
-                            raise NoSuchValueAliasException(
-                                alias=rest,
-                                msg=f"Can't retrive value for alias '{rest}': no such alias registered.",
-                            )
-                    else:
-                        raise Exception(
-                            f"Can't retrieve value for '{value_id}': invalid reference type '{ref_type}'."
-                        )
+                    _value_id = self._alias_resolver.resolve_alias(value)
         else:
-            _value_id = value_id
+            _value_id = value
 
         assert _value_id is not None
 
         if _value_id in self._registered_values.keys():
-            value = self._registered_values[_value_id]
-            return value
+            _value = self._registered_values[_value_id]
+            return _value
 
         matches = []
         for store_id, store in self.data_archives.items():
@@ -338,12 +386,12 @@ class DataRegistry(object):
 
         if len(matches) == 0:
             raise NoSuchValueIdException(
-                value_id=_value_id, msg=f"No value registered with id: {value_id}"
+                value_id=_value_id, msg=f"No value registered with id: {value}"
             )
         elif len(matches) > 1:
             raise NoSuchValueIdException(
                 value_id=_value_id,
-                msg=f"Found value with id '{value_id}' in multiple archives, this is not supported (yet): {matches}",
+                msg=f"Found value with id '{value}' in multiple archives, this is not supported (yet): {matches}",
             )
 
         self._value_archive_lookup_map[_value_id] = matches[0]
@@ -356,46 +404,115 @@ class DataRegistry(object):
 
     def store_value(
         self,
-        value: Union[Value, uuid.UUID],
-        store_id: Optional[str] = None,
-    ) -> Optional[PersistedData]:
+        value: Union[ValueLink, uuid.UUID, str],
+        store_id: Union[str, None] = None,
+    ) -> Union[PersistedData, None]:
 
         if store_id is None:
             store_id = self.default_data_store
 
-        if isinstance(value, uuid.UUID):
-            value = self.get_value(value)
+        _value = self.get_value(value)
 
         store: DataStore = self.get_archive(archive_id=store_id)  # type: ignore
         if not isinstance(store, DataStore):
             raise Exception(f"Can't store value into store '{store_id}': not writable.")
 
         # make sure all property values are available
-        if value.pedigree != ORPHAN:
-            for value_id in value.pedigree.inputs.values():
+        if _value.pedigree != ORPHAN:
+            for value_id in _value.pedigree.inputs.values():
                 self.store_value(value=value_id, store_id=store_id)
 
-        if not store.has_value(value.value_id):
-            event = ValuePreStoreEvent.construct(kiara_id=self._kiara.id, value=value)
+        if not store.has_value(_value.value_id):
+            event = ValuePreStoreEvent.construct(kiara_id=self._kiara.id, value=_value)
             self._event_callback(event)
-            persisted_value = store.store_value(value)
-            value._is_stored = True
-            self._value_archive_lookup_map[value.value_id] = store_id
-            self._persisted_value_descs[value.value_id] = persisted_value
-            property_values = value.property_values
+            persisted_value = store.store_value(_value)
+            _value._is_stored = True
+            self._value_archive_lookup_map[_value.value_id] = store_id
+            self._persisted_value_descs[_value.value_id] = persisted_value
+            property_values = _value.property_values
 
             for property, property_value in property_values.items():
                 self.store_value(value=property_value, store_id=store_id)
         else:
             persisted_value = None
 
-        store_event = ValueStoredEvent.construct(kiara_id=self._kiara.id, value=value)
+        store_event = ValueStoredEvent.construct(kiara_id=self._kiara.id, value=_value)
         self._event_callback(store_event)
 
         return persisted_value
 
+    def lookup_aliases(self, value: Union[Value, uuid.UUID]) -> Set[str]:
+
+        if isinstance(value, Value):
+            value = value.value_id
+
+        return self._kiara.alias_registry.find_aliases_for_value_id(value_id=value)
+
+    def create_value_info(self, value: Union[Value, uuid.UUID]) -> ValueInfo:
+
+        if isinstance(value, uuid.UUID):
+            value = self.get_value(value=value)
+
+        value_info = ValueInfo.create_from_instance(kiara=self._kiara, instance=value)
+        return value_info
+
+    def find_values(self, matcher: ValueMatcher) -> Dict[uuid.UUID, Value]:
+
+        matches: Dict[uuid.UUID, Value] = {}
+        for store_id, store in self.data_archives.items():
+            try:
+                _matches = store.find_values(matcher=matcher)
+                for value in _matches:
+                    if value.value_id in matches.keys():
+                        raise Exception(
+                            f"Found value '{value.value_id}' multiple times, this is not supported yet."
+                        )
+                    self._value_archive_lookup_map[value.value_id] = store_id
+                    value._set_registry(self)
+                    value._is_stored = True
+                    self._registered_values[value.value_id] = value
+                    matches[value.value_id] = value
+                    self._values_by_hash.setdefault(value.value_hash, set()).add(
+                        value.value_id
+                    )
+            except NotImplementedError:
+                all_value_ids = store.value_ids
+                if all_value_ids is None:
+                    continue
+                for value_id in all_value_ids:
+                    value = store.retrieve_value(value_id=value_id)
+                    value._set_registry(self)
+                    value._is_stored = True
+
+                    self._registered_values[value.value_id] = value
+
+                    match = matcher.is_match(value, kiara=self._kiara)
+                    if match:
+                        if value.value_id in matches.keys():
+                            raise Exception(
+                                f"Found value '{value.value_id}' multiple times, this is not supported yet."
+                            )
+                        matches[value.value_id] = value
+
+        return matches
+
+    def find_values_with_aliases(self, matcher: ValueMatcher) -> Dict[str, Value]:
+
+        matcher = matcher.copy(update={"has_aliases": True})
+        all_values = self.find_values(matcher)
+        result = {}
+        for value in all_values.values():
+            aliases = self._kiara.alias_registry.find_aliases_for_value_id(
+                value_id=value.value_id
+            )
+            for a in aliases:
+                assert a not in result  # this is a bug
+                result[a] = value
+
+        return result
+
     def find_values_for_hash(
-        self, value_hash: str, data_type_name: Optional[str] = None
+        self, value_hash: str, data_type_name: Union[str, None] = None
     ) -> Set[Value]:
 
         if data_type_name:
@@ -423,7 +540,7 @@ class DataRegistry(object):
             if stored:
                 self._values_by_hash[value_hash] = stored
 
-        return set((self.get_value(value_id=v_id) for v_id in stored))
+        return set((self.get_value(value=v_id) for v_id in stored))
 
     def find_destinies_for_value(
         self, value_id: uuid.UUID, alias_filter: str = None
@@ -434,8 +551,8 @@ class DataRegistry(object):
 
         all_destinies: Dict[str, uuid.UUID] = {}
         for archive_id, archive in self._data_archives.items():
-            destinies: Optional[
-                Mapping[str, uuid.UUID]
+            destinies: Union[
+                Mapping[str, uuid.UUID], None
             ] = archive.find_destinies_for_value(
                 value_id=value_id, alias_filter=alias_filter
             )
@@ -452,7 +569,7 @@ class DataRegistry(object):
         self,
         data: Any,
         schema: Union[ValueSchema, str] = None,
-        pedigree: Optional[ValuePedigree] = None,
+        pedigree: Union[ValuePedigree, None] = None,
         pedigree_output_name: str = None,
         reuse_existing: bool = True,
     ) -> Value:
@@ -476,11 +593,11 @@ class DataRegistry(object):
         return value
 
     def _find_existing_value(
-        self, data: Any, schema: Optional[ValueSchema]
+        self, data: Any, schema: Union[ValueSchema, None]
     ) -> Tuple[
-        Optional[Value],
+        Union[Value, None],
         DataType,
-        Optional[Any],
+        Union[Any, None],
         Union[str, SerializedData],
         ValueStatus,
         str,
@@ -513,7 +630,7 @@ class DataRegistry(object):
             # return data
 
         try:
-            value = self.get_value(value_id=data)
+            value = self.get_value(value=data)
             if value.is_serializable:
                 serialized = value.serialized_data
             else:
@@ -543,7 +660,7 @@ class DataRegistry(object):
             data=data, schema=schema
         )
 
-        existing_value: Optional[Value] = None
+        existing_value: Union[Value, None] = None
         if value_hash != INVALID_HASH_MARKER:
             existing = self.find_values_for_hash(value_hash=value_hash)
             if existing:
@@ -584,7 +701,7 @@ class DataRegistry(object):
         self,
         data: Any,
         schema: Union[None, str, ValueSchema] = None,
-        pedigree: Optional[ValuePedigree] = None,
+        pedigree: Union[ValuePedigree, None] = None,
         pedigree_output_name: str = None,
         reuse_existing: bool = True,
     ) -> Tuple[Value, bool]:
@@ -608,24 +725,20 @@ class DataRegistry(object):
                 f"Can't register data of type '{schema.type}': type not registered. Available types: {', '.join(self._kiara.data_type_names)}"
             )
 
-        if schema.default in [None, SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
-            if data is None or data is SpecialValue.NO_VALUE:
-                return (self.NONE_VALUE, False)
-            elif data is SpecialValue.NOT_SET:
-                return (self.NOT_SET_VALUE, False)
-        else:
-            # TODO: allow other value_ids in defaults?
-            if data in [None, SpecialValue.NO_VALUE, SpecialValue.NOT_SET]:
-                if callable(schema.default):
-                    data = schema.default()
-                else:
-                    data = copy.deepcopy(schema.default)
+        if data is SpecialValue.NOT_SET and schema.default is not SpecialValue.NOT_SET:
+            if callable(schema.default):
+                raise NotImplementedError()
+                data = schema.default()
+            else:
+                data = copy.deepcopy(schema.default)
 
-        # data_type: Optional[DataType] = None
-        # status: Optional[ValueStatus] = None
-        # value_hash: Optional[str] = None
+        if data is None:
+            data = SpecialValue.NO_VALUE
 
-        if reuse_existing:
+        if reuse_existing and data not in [
+            SpecialValue.NO_VALUE,
+            SpecialValue.NOT_SET,
+        ]:
             (
                 _existing,
                 data_type,
@@ -640,7 +753,6 @@ class DataRegistry(object):
                 # TODO: check pedigree
                 return (_existing, False)
         else:
-
             data_type = self._kiara.type_registry.retrieve_data_type(
                 data_type_name=schema.type, data_type_config=schema.type_config
             )
@@ -679,6 +791,7 @@ class DataRegistry(object):
             kiara_id=self._kiara.id,
             pedigree_output_name=pedigree_output_name,
         )
+
         ID_REGISTRY.update_metadata(v_id, obj=value)
         value._data_registry = self
 
@@ -722,7 +835,7 @@ class DataRegistry(object):
 
     def retrieve_serialized_value(
         self, value_id: uuid.UUID
-    ) -> Optional[SerializedData]:
+    ) -> Union[SerializedData, None]:
         """Create a LoadConfig object from the details of the persisted version of this value."""
 
         pv = self.retrieve_persisted_value_details(value_id=value_id)
@@ -734,7 +847,7 @@ class DataRegistry(object):
     def retrieve_chunk(
         self,
         chunk_id: str,
-        archive_id: Optional[uuid.UUID] = None,
+        archive_id: Union[uuid.UUID, None] = None,
         as_file: Union[None, bool, str] = None,
         symlink_ok: bool = True,
     ) -> Union[str, bytes]:
@@ -748,11 +861,11 @@ class DataRegistry(object):
         return chunk
 
     def retrieve_value_data(
-        self, value: Union[uuid.UUID, Value], target_profile: Optional[str] = None
+        self, value: Union[uuid.UUID, Value], target_profile: Union[str, None] = None
     ) -> Any:
 
         if isinstance(value, uuid.UUID):
-            value = self.get_value(value_id=value)
+            value = self.get_value(value=value)
 
         if value.value_id in self._cached_data.keys():
             return self._cached_data[value.value_id]
@@ -779,24 +892,32 @@ class DataRegistry(object):
         module = self._kiara.create_module(manifest=manifest)
         op = Operation.create_from_module(module=module)
 
-        input_field_match: Optional[str] = None
+        input_field_match: Union[str, None] = None
+
         if len(op.inputs_schema) == 1:
             input_field_match = next(iter(op.inputs_schema.keys()))
         else:
             for input_field, schema in op.inputs_schema.items():
-                if schema.type == value.data_type_name:
-                    if input_field_match is not None:
-                        raise Exception(
-                            f"Can't determine input field for deserialization operation '{module.module_type_name}': multiple input fields with type '{input_field_match}'."
-                        )
-                    else:
-                        input_field_match = input_field
+                for dt in self._kiara.type_registry.get_type_lineage(
+                    value.data_type_name
+                ):
+                    if schema.type == dt:
+                        if input_field_match is not None:
+                            raise Exception(
+                                f"Can't determine input field for deserialization operation '{module.module_type_name}': multiple input fields with type '{input_field_match}'."
+                            )
+                        else:
+                            input_field_match = input_field
+                            break
+                if input_field_match:
+                    break
+
         if input_field_match is None:
             raise Exception(
                 f"Can't determine input field for deserialization operation '{module.module_type_name}'."
             )
 
-        result_field_match: Optional[str] = None
+        result_field_match: Union[str, None] = None
         for result_field, schema in op.outputs_schema.items():
             if schema.type == "python_object":
                 if result_field_match is not None:
@@ -823,7 +944,7 @@ class DataRegistry(object):
 
         return parsed
 
-    def load_values(self, values: Mapping[str, Optional[uuid.UUID]]) -> ValueMap:
+    def load_values(self, values: Mapping[str, Union[uuid.UUID, None]]) -> ValueMap:
 
         value_items = {}
         schemas = {}
@@ -831,13 +952,15 @@ class DataRegistry(object):
             if value_id is None:
                 value_id = NONE_VALUE_ID
 
-            value = self.get_value(value_id=value_id)
+            value = self.get_value(value=value_id)
             value_items[field_name] = value
             schemas[field_name] = value.value_schema
 
         return ValueMapReadOnly(value_items=value_items, values_schema=schemas)
 
-    def load_data(self, values: Mapping[str, Optional[uuid.UUID]]) -> Mapping[str, Any]:
+    def load_data(
+        self, values: Mapping[str, Union[uuid.UUID, None]]
+    ) -> Mapping[str, Any]:
 
         result_values = self.load_values(values=values)
         return {k: v.data for k, v in result_values.items()}
@@ -862,11 +985,32 @@ class DataRegistry(object):
                 )
 
         values = {}
-
         failed = {}
-        for input_name, details in input_details.items():
 
-            value_schema = details["schema"]
+        _resolved: Dict[str, Value] = {}
+        _unresolved = {}
+        for input_name, details in input_details.items():
+            _d = data.get(input_name, SpecialValue.NOT_SET)
+            if isinstance(_d, str):
+                try:
+                    _d = uuid.UUID(_d)
+                except Exception:
+                    pass
+
+            if isinstance(_d, Value):
+                _resolved[input_name] = _d
+            elif isinstance(_d, uuid.UUID):
+                _resolved[input_name] = self.get_value(_d)
+            else:
+                _unresolved[input_name] = _d
+
+        for input_name, _value in _resolved.items():
+            # TODO: validate values against schema
+            values[input_name] = _value
+
+        for input_name, _data in _unresolved.items():
+
+            value_schema = input_details[input_name]["schema"]
 
             if input_name not in data.keys():
                 value_data = SpecialValue.NOT_SET
@@ -883,9 +1027,6 @@ class DataRegistry(object):
                 value = self.register_data(
                     data=value_data, schema=value_schema, reuse_existing=True
                 )
-                # value = self.retrieve_or_create_value(
-                #     value_data, value_schema=value_schema
-                # )
                 values[input_name] = value
             except Exception as e:
 
@@ -894,6 +1035,7 @@ class DataRegistry(object):
                 msg: Any = str(e)
                 if not msg:
                     msg = e
+
                 log_message("invalid.valueset", error_reason=msg, input_name=input_name)
                 failed[input_name] = e
 
