@@ -9,9 +9,10 @@ import abc
 import structlog
 import uuid
 from rich.console import RenderableType
-from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Optional, Set, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Set, Union
 
 from kiara.models.runtime_environment import RuntimeEnvironment
+from kiara.models.values.matchers import ValueMatcher
 from kiara.models.values.value import PersistedData, Value, ValuePedigree
 from kiara.models.values.value_schema import ValueSchema
 from kiara.registries import ARCHIVE_CONFIG_CLS, BaseArchive
@@ -43,7 +44,7 @@ class DataArchive(BaseArchive):
 
         if isinstance(value, Value):
             value_id: uuid.UUID = value.value_id
-            _value: Optional[Value] = value
+            _value: Union[Value, None] = value
         else:
             value_id = value
             _value = None
@@ -88,7 +89,7 @@ class DataArchive(BaseArchive):
             environment_hashes=value_data.get("environment_hashes", {}),
             pedigree=pedigree,
             pedigree_output_name=value_data["pedigree_output_name"],
-            data_type_class=value_data["data_type_class"],
+            data_type_info=value_data["data_type_info"],
             property_links=value_data["property_links"],
             destiny_backlinks=value_data["destiny_backlinks"],
         )
@@ -101,13 +102,12 @@ class DataArchive(BaseArchive):
         pass
 
     @property
-    def value_ids(self) -> Iterable[uuid.UUID]:
+    def value_ids(self) -> Union[None, Iterable[uuid.UUID]]:
         return self._retrieve_all_value_ids()
 
-    @abc.abstractmethod
     def _retrieve_all_value_ids(
-        self, data_type_name: Optional[str] = None
-    ) -> Iterable[uuid.UUID]:
+        self, data_type_name: Union[str, None] = None
+    ) -> Union[None, Iterable[uuid.UUID]]:
         pass
 
     def has_value(self, value_id: uuid.UUID) -> bool:
@@ -122,7 +122,10 @@ class DataArchive(BaseArchive):
             whether this data store contains the value with the specified id
         """
 
-        return value_id in self._retrieve_all_value_ids()
+        all_value_ids = self.value_ids
+        if all_value_ids is None:
+            return False
+        return value_id in all_value_ids
 
     def retrieve_environment_details(
         self, env_type: str, env_hash: str
@@ -147,11 +150,14 @@ class DataArchive(BaseArchive):
     ) -> Mapping[str, Any]:
         pass
 
+    def find_values(self, matcher: ValueMatcher) -> Iterable[Value]:
+        raise NotImplementedError()
+
     def find_values_with_hash(
         self,
         value_hash: str,
-        value_size: Optional[int] = None,
-        data_type_name: Optional[str] = None,
+        value_size: Union[int, None] = None,
+        data_type_name: Union[str, None] = None,
     ) -> Set[uuid.UUID]:
 
         if data_type_name is not None:
@@ -161,7 +167,7 @@ class DataArchive(BaseArchive):
             raise NotImplementedError()
 
         if value_hash in self._value_hash_index.keys():
-            value_ids: Optional[Set[uuid.UUID]] = self._value_hash_index[value_hash]
+            value_ids: Union[Set[uuid.UUID], None] = self._value_hash_index[value_hash]
         else:
             value_ids = self._find_values_with_hash(
                 value_hash=value_hash, data_type_name=data_type_name
@@ -177,14 +183,14 @@ class DataArchive(BaseArchive):
     def _find_values_with_hash(
         self,
         value_hash: str,
-        value_size: Optional[int] = None,
-        data_type_name: Optional[str] = None,
-    ) -> Optional[Set[uuid.UUID]]:
+        value_size: Union[int, None] = None,
+        data_type_name: Union[str, None] = None,
+    ) -> Union[Set[uuid.UUID], None]:
         pass
 
     def find_destinies_for_value(
-        self, value_id: uuid.UUID, alias_filter: Optional[str] = None
-    ) -> Optional[Mapping[str, uuid.UUID]]:
+        self, value_id: uuid.UUID, alias_filter: Union[str, None] = None
+    ) -> Union[Mapping[str, uuid.UUID], None]:
 
         return self._find_destinies_for_value(
             value_id=value_id, alias_filter=alias_filter
@@ -192,8 +198,8 @@ class DataArchive(BaseArchive):
 
     @abc.abstractmethod
     def _find_destinies_for_value(
-        self, value_id: uuid.UUID, alias_filter: Optional[str] = None
-    ) -> Optional[Mapping[str, uuid.UUID]]:
+        self, value_id: uuid.UUID, alias_filter: Union[str, None] = None
+    ) -> Union[Mapping[str, uuid.UUID], None]:
         pass
 
     @abc.abstractmethod
@@ -304,14 +310,23 @@ class BaseDataStore(DataStore):
     def _persist_value(self, value: Value) -> PersistedData:
 
         # TODO: check if value id is already persisted?
-        persisted_value_info: PersistedData = self._persist_value_data(value=value)
-        if not persisted_value_info:
-            raise Exception(
-                "Can't write persisted value info, no load config returned when persisting value."
-            )
-        if not isinstance(persisted_value_info, PersistedData):
-            raise Exception(
-                f"Can't write persisted value info, invalid result type '{type(persisted_value_info)}' when persisting value."
+        if value.is_set:
+            persisted_value_info: PersistedData = self._persist_value_data(value=value)
+            if not persisted_value_info:
+                raise Exception(
+                    "Can't write persisted value info, no load config returned when persisting value."
+                )
+            if not isinstance(persisted_value_info, PersistedData):
+                raise Exception(
+                    f"Can't write persisted value info, invalid result type '{type(persisted_value_info)}' when persisting value."
+                )
+        else:
+            persisted_value_info = PersistedData(
+                archive_id=self.archive_id,
+                data_type=value.data_type_name,
+                serialization_profile="none",
+                data_type_config=value.data_type_config,
+                chunk_id_map={},
             )
 
         self._persist_stored_value_info(
@@ -349,13 +364,17 @@ class BaseDataStore(DataStore):
         from kiara.utils.output import create_renderable_from_values
 
         all_values = {}
-        for value_id in self.value_ids:
+        all_value_ids = self.value_ids
+        if all_value_ids:
+            for value_id in all_value_ids:
 
-            value = self.kiara_context.data_registry.get_value(value_id)
-            all_values[str(value_id)] = value
-        table = create_renderable_from_values(values=all_values, config=config)
+                value = self.kiara_context.data_registry.get_value(value_id)
+                all_values[str(value_id)] = value
+            table = create_renderable_from_values(values=all_values, config=config)
 
-        return table
+            return table
+        else:
+            return "Data archive does not support statically determined ids."
 
 
 # class PersistenceMgmt(object):
