@@ -9,9 +9,10 @@ import abc
 import structlog
 import uuid
 from bidict import bidict
-from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Optional, Type
+from typing import TYPE_CHECKING, Any, Dict, Iterable, Mapping, Type, Union
 
 from kiara.context.config import JobCacheStrategy
+from kiara.exceptions import FailedJobException
 from kiara.models.events import KiaraEvent
 from kiara.models.events.job_registry import (
     JobArchiveAddedEvent,
@@ -24,6 +25,7 @@ from kiara.models.values.value import ValueMap
 from kiara.processing import ModuleProcessor
 from kiara.processing.synchronous import SynchronousProcessor
 from kiara.registries import BaseArchive
+from kiara.utils import get_dev_config, is_develop
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
@@ -43,7 +45,9 @@ class JobArchive(BaseArchive):
 
     @abc.abstractmethod
     def retrieve_all_job_hashes(
-        self, manifest_hash: Optional[str] = None, inputs_hash: Optional[str] = None
+        self,
+        manifest_hash: Union[str, None] = None,
+        inputs_hash: Union[str, None] = None,
     ) -> Iterable[str]:
         """Retrieve a list of all job record hashes (cids) that match the given filter arguments.
 
@@ -53,10 +57,10 @@ class JobArchive(BaseArchive):
         """
 
     @abc.abstractmethod
-    def _retrieve_record_for_job_hash(self, job_hash: str) -> Optional[JobRecord]:
+    def _retrieve_record_for_job_hash(self, job_hash: str) -> Union[JobRecord, None]:
         pass
 
-    def retrieve_record_for_job_hash(self, job_hash: str) -> Optional[JobRecord]:
+    def retrieve_record_for_job_hash(self, job_hash: str) -> Union[JobRecord, None]:
 
         job_record = self._retrieve_record_for_job_hash(job_hash=job_hash)
         return job_record
@@ -74,17 +78,23 @@ class JobMatcher(abc.ABC):
         self._kiara: Kiara = kiara
 
     @abc.abstractmethod
-    def find_existing_job(self, inputs_manifest: InputsManifest) -> Optional[JobRecord]:
+    def find_existing_job(
+        self, inputs_manifest: InputsManifest
+    ) -> Union[JobRecord, None]:
         pass
 
 
 class NoneJobMatcher(JobMatcher):
-    def find_existing_job(self, inputs_manifest: InputsManifest) -> Optional[JobRecord]:
+    def find_existing_job(
+        self, inputs_manifest: InputsManifest
+    ) -> Union[JobRecord, None]:
         return None
 
 
 class ValueIdJobMatcher(JobMatcher):
-    def find_existing_job(self, inputs_manifest: InputsManifest) -> Optional[JobRecord]:
+    def find_existing_job(
+        self, inputs_manifest: InputsManifest
+    ) -> Union[JobRecord, None]:
 
         matches = []
 
@@ -110,7 +120,9 @@ class ValueIdJobMatcher(JobMatcher):
 
 
 class DataHashJobMatcher(JobMatcher):
-    def find_existing_job(self, inputs_manifest: InputsManifest) -> Optional[JobRecord]:
+    def find_existing_job(
+        self, inputs_manifest: InputsManifest
+    ) -> Union[JobRecord, None]:
 
         matches = []
 
@@ -178,7 +190,7 @@ class JobRegistry(object):
         self._processor: ModuleProcessor = SynchronousProcessor(kiara=self._kiara)
         self._processor.register_job_status_listener(self)
         self._job_archives: Dict[str, JobArchive] = {}
-        self._default_job_store: Optional[str] = None
+        self._default_job_store: Union[str, None] = None
 
         self._event_callback = self._kiara.event_registry.add_producer(self)
 
@@ -192,6 +204,15 @@ class JobRegistry(object):
     def job_matcher(self) -> JobMatcher:
 
         strategy = self._kiara.runtime_config.job_cache
+        if is_develop():
+            dev_config = get_dev_config()
+            if not dev_config.job_cache:
+                logger.debug(
+                    "disable.job_cache",
+                    reason="dev mode enabled and 'disable_job_cache' is set.",
+                )
+                strategy = JobCacheStrategy.no_cache
+
         job_matcher = self._job_matcher_cache.get(strategy, None)
         if job_matcher is None:
             if strategy == JobCacheStrategy.no_cache:
@@ -210,7 +231,7 @@ class JobRegistry(object):
 
         return [JobArchiveAddedEvent, JobRecordPreStoreEvent, JobRecordStoredEvent]
 
-    def register_job_archive(self, archive: JobArchive, alias: Optional[str] = None):
+    def register_job_archive(self, archive: JobArchive, alias: Union[str, None] = None):
 
         if alias is None:
             alias = str(archive.archive_id)
@@ -245,7 +266,7 @@ class JobRegistry(object):
             raise Exception("No default job store set (yet).")
         return self._default_job_store  # type: ignore
 
-    def get_archive(self, store_id: Optional[str] = None) -> JobArchive:
+    def get_archive(self, store_id: Union[str, None] = None) -> JobArchive:
 
         if store_id is None:
             store_id = self.default_job_store
@@ -259,7 +280,10 @@ class JobRegistry(object):
         return self._job_archives
 
     def job_status_changed(
-        self, job_id: uuid.UUID, old_status: Optional[JobStatus], new_status: JobStatus
+        self,
+        job_id: uuid.UUID,
+        old_status: Union[JobStatus, None],
+        new_status: JobStatus,
     ):
 
         # print(f"JOB STATUS CHANGED: {job_id} - {old_status} - {new_status.value}")
@@ -323,6 +347,24 @@ class JobRegistry(object):
 
         return self._processor.get_job_record(job_id)
 
+    def get_job_record(self, job_id: uuid.UUID) -> Union[JobRecord, None]:
+
+        if job_id in self._archived_records.keys():
+            return self._archived_records[job_id]
+
+        try:
+            job_record = self._processor.get_job_record(job_id=job_id)
+            return job_record
+        except Exception:
+            pass
+
+        job = self._processor.get_job(job_id=job_id)
+        if job is not None:
+            if job.status == JobStatus.FAILED:
+                return None
+
+        raise NotImplementedError()
+
     def retrieve_all_job_records(self) -> Mapping[str, JobRecord]:
 
         all_records: Dict[str, JobRecord] = {}
@@ -340,7 +382,7 @@ class JobRegistry(object):
 
     def find_matching_job_record(
         self, inputs_manifest: InputsManifest
-    ) -> Optional[uuid.UUID]:
+    ) -> Union[uuid.UUID, None]:
         """Check if a job with same inputs manifest already ran some time before.
 
         Arguments:
@@ -359,13 +401,27 @@ class JobRegistry(object):
             job_id = self._finished_jobs[inputs_manifest.job_hash]
             return job_id
 
+        module = self._kiara.module_registry.create_module(manifest=inputs_manifest)
+        if not module.characteristics.is_idempotent:
+            log.debug(
+                "skip.job_matching",
+                reason="module is not idempotent",
+                module_type=inputs_manifest.module_type,
+            )
+            return None
+
         job_record = self.job_matcher.find_existing_job(inputs_manifest=inputs_manifest)
         if job_record is None:
             return None
 
         self._finished_jobs[inputs_manifest.job_hash] = job_record.job_id
         self._archived_records[job_record.job_id] = job_record
-        log.debug("job.use_cached")
+        log.debug(
+            "job.found_cached_record",
+            job_id=str(job_record.job_id),
+            job_hash=inputs_manifest.job_hash,
+            module_type=inputs_manifest.module_type,
+        )
         return job_record.job_id
 
     def prepare_job_config(
@@ -373,6 +429,7 @@ class JobRegistry(object):
     ) -> JobConfig:
 
         module = self._kiara.create_module(manifest=manifest)
+
         job_config = JobConfig.create_from_module(
             data_registry=self._kiara.data_registry, module=module, inputs=inputs
         )
@@ -380,13 +437,22 @@ class JobRegistry(object):
         return job_config
 
     def execute(
-        self, manifest: Manifest, inputs: Mapping[str, Any], wait: bool = False
+        self,
+        manifest: Manifest,
+        inputs: Mapping[str, Any],
+        wait: bool = False,
+        job_metadata: Union[None, Any] = None,
     ) -> uuid.UUID:
 
         job_config = self.prepare_job_config(manifest=manifest, inputs=inputs)
-        return self.execute_job(job_config, wait=wait)
+        return self.execute_job(job_config, wait=wait, job_metadata=job_metadata)
 
-    def execute_job(self, job_config: JobConfig, wait: bool = False) -> uuid.UUID:
+    def execute_job(
+        self,
+        job_config: JobConfig,
+        wait: bool = False,
+        job_metadata: Union[None, Any] = None,
+    ) -> uuid.UUID:
 
         log = logger.bind(
             module_type=job_config.module_type,
@@ -397,11 +463,28 @@ class JobRegistry(object):
 
         stored_job = self.find_matching_job_record(inputs_manifest=job_config)
         if stored_job is not None:
+            log.debug(
+                "job.use_cached",
+                job_id=str(stored_job),
+                module_type=job_config.module_type,
+            )
             return stored_job
 
-        log.debug("job.execute")
+        if job_metadata is None:
+            job_metadata = {}
 
-        job_id = self._processor.create_job(job_config=job_config)
+        is_pipeline_step = job_metadata.get("is_pipeline_step", False)
+        dbg_data = {
+            "module_type": job_config.module_type,
+            "is_pipeline_step": is_pipeline_step,
+        }
+        if is_pipeline_step:
+            dbg_data["step_id"] = job_metadata["step_id"]
+        log.debug("job.execute", **dbg_data)
+
+        job_id = self._processor.create_job(
+            job_config=job_config, job_metadata=job_metadata
+        )
         self._active_jobs[job_config.job_hash] = job_id
 
         try:
@@ -431,10 +514,13 @@ class JobRegistry(object):
                     msg = str(job._exception)
                     if not msg:
                         msg = repr(job._exception)
-                raise Exception(f"Job failed: {msg}")
+                raise FailedJobException(job=job, msg=msg)
             else:
                 raise Exception(f"Can't retrieve job with id '{job_id}': no such job.")
 
+    def get_job(self, job_id: uuid.UUID) -> ActiveJob:
+        return self._processor.get_job(job_id=job_id)
+
     def get_job_status(self, job_id: uuid.UUID) -> JobStatus:
 
         if job_id in self._archived_records.keys():
