@@ -4,16 +4,20 @@
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
+import datetime
 import orjson
 import shutil
+import structlog
 import uuid
 from pathlib import Path
-from typing import Any, Dict, Iterable, Optional
+from typing import Any, Dict, Iterable, Union
 
 from kiara.models.module.jobs import JobRecord
 from kiara.registries import ArchiveDetails, FileSystemArchiveConfig
 from kiara.registries.jobs import MANIFEST_SUB_PATH, JobArchive, JobStore
 
+log = structlog.getLogger()
+
 
 class FileSystemJobArchive(JobArchive):
 
@@ -31,7 +35,7 @@ class FileSystemJobArchive(JobArchive):
     def __init__(self, archive_id: uuid.UUID, config: FileSystemArchiveConfig):
 
         super().__init__(archive_id=archive_id, config=config)
-        self._base_path: Optional[Path] = None
+        self._base_path: Union[Path, None] = None
 
     def get_archive_details(self) -> ArchiveDetails:
 
@@ -55,7 +59,9 @@ class FileSystemJobArchive(JobArchive):
         shutil.rmtree(self.job_store_path)
 
     def retrieve_all_job_hashes(
-        self, manifest_hash: Optional[str] = None, inputs_hash: Optional[str] = None
+        self,
+        manifest_hash: Union[str, None] = None,
+        inputs_hash: Union[str, None] = None,
     ) -> Iterable[str]:
 
         base_path = self.job_store_path / MANIFEST_SUB_PATH
@@ -75,7 +81,7 @@ class FileSystemJobArchive(JobArchive):
             result.append(record.name[0:-11])
         return result
 
-    def _retrieve_record_for_job_hash(self, job_hash: str) -> Optional[JobRecord]:
+    def _retrieve_record_for_job_hash(self, job_hash: str) -> Union[JobRecord, None]:
 
         base_path = self.job_store_path / MANIFEST_SUB_PATH
         records = list(base_path.glob(f"*/*/{job_hash}.job_record"))
@@ -156,13 +162,26 @@ class FileSystemJobStore(FileSystemJobArchive, JobStore):
         job_folder = manifest_folder / inputs_hash
         job_folder.mkdir(parents=True, exist_ok=True)
 
-        job_details_file_name = job_folder / f"{job_record.job_hash}.job_record"
-        if job_details_file_name.exists():
-            raise Exception(
-                f"Job record already exists: {job_details_file_name.as_posix()}"
+        job_details_file = job_folder / f"{job_record.job_hash}.job_record"
+        exists = False
+        if job_details_file.exists():
+            exists = True
+            # TODO: check details match? or overwrite
+            file_m_time = datetime.datetime.fromtimestamp(
+                job_details_file.stat().st_mtime
+            ).timestamp()
+            archive = job_folder / ".archive"
+            archive.mkdir(parents=True, exist_ok=True)
+            backup = archive / f"{job_details_file.name}.{file_m_time}"
+            log.debug(
+                "overwrite.store_job_record",
+                reason="job record already exists",
+                job_hash=job_record.job_hash,
+                new_path=backup.as_posix(),
             )
+            shutil.move(job_details_file.as_posix(), backup)
 
-        job_details_file_name.write_text(job_record.json())
+        job_details_file.write_text(job_record.json())
 
         for output_name, output_v_id in job_record.outputs.items():
 
@@ -170,7 +189,7 @@ class FileSystemJobStore(FileSystemJobArchive, JobStore):
                 job_folder / f"output__{output_name}__value_id__{output_v_id}.json"
             )
 
-            if outputs_file_name.exists():
+            if outputs_file_name.exists() and not exists:
                 # if value.pedigree_output_name == "__void__":
                 #     return
                 # else:
