@@ -6,21 +6,14 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 from bidict import bidict
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    Iterable,
-    List,
-    Mapping,
-    Optional,
-    Set,
-    Type,
-)
+from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Set, Type, Union
 
 from kiara.data_types import DataType
 from kiara.defaults import KIARA_ROOT_TYPE_NAME
-from kiara.models.values.data_type import DataTypeClassesInfo, DataTypeClassInfo
+from kiara.interfaces.python_api.models.info import (
+    DataTypeClassesInfo,
+    DataTypeClassInfo,
+)
 from kiara.utils.class_loading import find_all_data_types
 
 if TYPE_CHECKING:
@@ -29,30 +22,18 @@ if TYPE_CHECKING:
     from kiara.context import Kiara
 
 
-TYPE_PROFILE_MAP = {
-    "csv_file": "file",
-    "text_file_bundle": "file_bundle",
-    "csv_file_bundle": "file_bundle",
-    "table": "table",
-    "graphml_file": "file",
-    "gexf_file": "file",
-    "gml_file": "file",
-    "shp_file": "file",
-}
-
-
 class TypeRegistry(object):
     def __init__(self, kiara: "Kiara"):
 
         self._kiara: Kiara = kiara
-        self._data_types: Optional[bidict[str, Type[DataType]]] = None
+        self._data_types: Union[bidict[str, Type[DataType]], None] = None
         self._data_type_metadata: Dict[str, DataTypeClassInfo] = {}
         self._cached_data_type_objects: Dict[int, DataType] = {}
         # self._registered_python_classes: Dict[Type, typing.List[str]] = None  # type: ignore
-        self._type_hierarchy: Optional[nx.DiGraph] = None
+        self._type_hierarchy: Union[nx.DiGraph, None] = None
         self._lineages_cache: Dict[str, List[str]] = {}
 
-        self._type_profiles: Optional[Dict[str, Mapping[str, Any]]] = None
+        self._type_profiles: Union[Dict[str, Mapping[str, Any]], None] = None
 
     def invalidate_types(self):
 
@@ -60,7 +41,9 @@ class TypeRegistry(object):
         # self._registered_python_classes = None
 
     def retrieve_data_type(
-        self, data_type_name: str, data_type_config: Optional[Mapping[str, Any]] = None
+        self,
+        data_type_name: str,
+        data_type_config: Union[Mapping[str, Any], None] = None,
     ) -> DataType:
 
         if data_type_config is None:
@@ -124,7 +107,7 @@ class TypeRegistry(object):
         if self._type_hierarchy is not None:
             return self._type_hierarchy
 
-        def recursive_base_find(cls: Type, current: Optional[List[str]] = None):
+        def recursive_base_find(cls: Type, current: Union[List[str], None] = None):
 
             if current is None:
                 current = []
@@ -181,7 +164,7 @@ class TypeRegistry(object):
         sub_graph = graph.subgraph(desc)
         return sub_graph
 
-    def get_type_lineage(self, data_type_name: str) -> Iterable[str]:
+    def get_type_lineage(self, data_type_name: str) -> List[str]:
         """Returns the shortest path between the specified type and the root, in reverse direction starting from the specified type."""
 
         if data_type_name not in self.data_type_profiles.keys():
@@ -209,6 +192,21 @@ class TypeRegistry(object):
         desc = nx.descendants(self.data_type_hierarchy, data_type_name)
         return desc
 
+    def is_profile(self, data_type_name: str) -> bool:
+
+        type_config = self.data_type_profiles.get(data_type_name, {}).get(
+            "type_config", None
+        )
+        return True if type_config else False
+
+    def get_profile_parent(self, data_type_name: str) -> Union[None, bool]:
+        """Return the parent data type of the specified data type (if that is indeed a profile name).
+
+        If the specified data type is not a profile name, 'None' will be returned.
+        """
+
+        return self.data_type_profiles.get(data_type_name, {}).get("type_name", None)
+
     def get_associated_profiles(
         self, data_type_name: str
     ) -> Mapping[str, Mapping[str, Any]]:
@@ -247,6 +245,17 @@ class TypeRegistry(object):
             )
         return t
 
+    def get_data_type_instance(
+        self, type_name: str, type_config: Union[None, Mapping[str, Any]] = None
+    ) -> DataType:
+
+        cls = self.get_data_type_cls(type_name=type_name)
+        if not type_config:
+            obj = cls()
+        else:
+            obj = cls(**type_config)
+        return obj
+
     def get_type_metadata(self, type_name: str) -> DataTypeClassInfo:
 
         md = self._data_type_metadata.get(type_name, None)
@@ -258,7 +267,7 @@ class TypeRegistry(object):
         return self._data_type_metadata[type_name]
 
     def get_context_metadata(
-        self, alias: Optional[str] = None, only_for_package: Optional[str] = None
+        self, alias: Union[str, None] = None, only_for_package: Union[str, None] = None
     ) -> DataTypeClassesInfo:
 
         result = {}
