@@ -6,7 +6,7 @@ from sqlalchemy import engine_from_config, pool
 
 # this is the Alembic Config object, which provides
 # access to the values within the .ini file in use.
-from kiara import Kiara
+from kiara.context import Kiara
 from kiara.context.orm import Base
 from kiara.defaults import KIARA_DB_MIGRATIONS_FOLDER
 
