@@ -9,6 +9,8 @@ import importlib
 import inspect
 import os
 import sys
+from pkgutil import iter_modules
+from rich.markdown import Markdown
 from stevedore import ExtensionManager
 from types import ModuleType
 from typing import (
@@ -19,7 +21,6 @@ from typing import (
     Iterable,
     List,
     Mapping,
-    Optional,
     Tuple,
     Type,
     TypeVar,
@@ -28,12 +29,13 @@ from typing import (
 
 from kiara.utils import (
     _get_all_subclasses,
-    _import_modules_recursively,
     camel_case_to_snake_case,
     is_debug,
+    is_develop,
     log_exception,
     log_message,
 )
+from kiara.utils.develop import log_dev_message
 
 if TYPE_CHECKING:
     from kiara.modules import KiaraModule
@@ -123,13 +125,13 @@ def find_subclasses_under(
 def _process_subclass(
     sub_class: Type,
     base_class: Type,
-    type_id_key: Optional[str],
-    type_id_func: Optional[Callable],
+    type_id_key: Union[str, None],
+    type_id_func: Union[Callable, None],
     type_id_no_attach: bool,
     attach_python_metadata: Union[bool, str] = False,
     ignore_abstract_classes: bool = True,
     ignore_modules_with_null_module_name: bool = True,
-) -> Optional[str]:
+) -> Union[str, None]:
     """Process subclasses of a base class that live under a module (recursively).
 
     Arguments:
@@ -147,6 +149,34 @@ def _process_subclass(
     """
     is_abstract = inspect.isabstract(sub_class)
     if ignore_abstract_classes and is_abstract:
+
+        if sub_class.__dict__.get("_is_abstract", False):
+            return None
+
+        if is_develop():
+            from kiara.modules import KiaraModule
+
+            if base_class == KiaraModule and is_develop():
+                missing = []
+                abs_meth = sub_class.__abstractmethods__
+                if "create_inputs_schema" in abs_meth:
+                    missing.append("create_inputs_schema")
+                if "create_outputs_schema" in abs_meth:
+                    missing.append("create_outputs_schema")
+                if not hasattr(sub_class, "process"):
+                    missing.append("process")
+
+                name = f"{sub_class.__module__}.{sub_class.__name__}"
+                title = "Invalid kiara module"
+                if hasattr(sub_class, "_module_type_name"):
+                    name = f"**{name}** ( *{sub_class._module_type_name}* )"  # type: ignore
+                    title = f"{title} '[i]{sub_class._module_type_name}[/i]'"  # type: ignore
+                msg = f"Invalid kiara module: {name}\n\nMissing method(s):"
+                for m in missing:
+                    msg = f"{msg}\n- *{m}*"
+
+                log_dev_message(msg=Markdown(msg), title=title)
+
         log_message(
             "ignore.subclass",
             sub_class=sub_class,
@@ -167,7 +197,7 @@ def _process_subclass(
                     "ignore.subclass",
                     sub_class=sub_class,
                     base_class=base_class,
-                    reason="'_module_type_name' subclass is set to 'None'",
+                    reason=f"'{ type_id_key }' subclass is set to 'None'",
                 )
                 return None
             if not type_id and not is_abstract:
@@ -199,7 +229,7 @@ def load_all_subclasses_for_entry_point(
     entry_point_name: str,
     base_class: Type[SUBCLASS_TYPE],
     ignore_abstract_classes: bool = True,
-    type_id_key: Optional[str] = None,
+    type_id_key: Union[str, None] = None,
     type_id_func: Callable = None,
     type_id_no_attach: bool = False,
     attach_python_metadata: Union[bool, str] = False,
@@ -284,10 +314,7 @@ def load_all_subclasses_for_entry_point(
                     args = plugin.plugin[1:]
                 classes = func(*args)
             except Exception as e:
-                if is_debug():
-                    import traceback
-
-                    traceback.print_exc()
+                log_exception(e)
                 raise Exception(f"Error trying to load plugin '{plugin.plugin}': {e}")
 
             for sub_class in classes:
@@ -350,8 +377,17 @@ def find_all_kiara_modules() -> Dict[str, Type["KiaraModule"]]:
     for k, cls in modules.items():
 
         if not hasattr(cls, "process"):
+            if is_develop():
+                msg = f"Invalid kiara module: **{cls.__module__}.{cls.__name__}**\n\nMissing method(s):\n- *process*"
+                log_dev_message(msg=Markdown(msg))
+
             # TODO: check signature of process method
-            log_message("ignore.module.class", cls=cls, reason="no 'process' method")
+            log_message(
+                "ignore.subclass",
+                sub_class=cls,
+                base_class=KiaraModule,
+                reason="'process' method is missing",
+            )
             continue
 
         result[k] = cls
@@ -504,7 +540,9 @@ def find_operations_under(
     )
 
 
-def find_pipeline_base_path_for_module(module: Union[str, ModuleType]) -> Optional[str]:
+def find_pipeline_base_path_for_module(
+    module: Union[str, ModuleType]
+) -> Union[str, None]:
 
     if hasattr(sys, "frozen"):
         raise NotImplementedError("Pyinstaller bundling not supported yet.")
@@ -525,7 +563,7 @@ def find_pipeline_base_path_for_module(module: Union[str, ModuleType]) -> Option
 
 def find_all_kiara_pipeline_paths(
     skip_errors: bool = False,
-) -> Dict[str, Optional[Mapping[str, Any]]]:
+) -> Dict[str, Union[Mapping[str, Any], None]]:
 
     import logging
 
@@ -544,7 +582,7 @@ def find_all_kiara_pipeline_paths(
         namespace="kiara.pipelines", invoke_on_load=False, propagate_map_exceptions=True
     )
 
-    paths: Dict[str, Optional[Mapping[str, Any]]] = {}
+    paths: Dict[str, Union[Mapping[str, Any], None]] = {}
     # TODO: make sure we load 'core' first?
     for plugin in mgr:
 
@@ -563,7 +601,7 @@ def find_all_kiara_pipeline_paths(
                     args = plugin.plugin[1:]
 
                 f_args = []
-                metadata: Optional[Mapping[str, Any]] = None
+                metadata: Union[Mapping[str, Any], None] = None
                 if len(args) >= 1:
                     f_args.append(args[0])
                 if len(args) >= 2:
@@ -588,10 +626,7 @@ def find_all_kiara_pipeline_paths(
                         paths[path] = metadata
 
             except Exception as e:
-                if is_debug():
-                    import traceback
-
-                    traceback.print_exc()
+                log_exception(e)
                 if skip_errors:
                     log_message(
                         "ignore.pipline_entrypoint", entrypoint_name=name, reason=str(e)
@@ -663,3 +698,23 @@ def find_all_cli_subcommands():
 #
 #     # TODO: typecheck?
 #     return _callable_wrapper(func=func)  # type: ignore
+def _import_modules_recursively(module: "ModuleType"):
+
+    if not hasattr(module, "__path__"):
+        return
+
+    for submodule in iter_modules(module.__path__):  # type: ignore
+
+        try:
+            submodule_mod = importlib.import_module(
+                f"{module.__name__}.{submodule.name}"
+            )
+            if hasattr(submodule_mod, "__path__"):
+                _import_modules_recursively(submodule_mod)
+        except Exception as e:
+            logger.error(
+                "ignore.python_module",
+                module=f"{module.__name__}.{submodule.name}",
+                reason=str(e),
+                base_module=str(module),
+            )
