@@ -7,34 +7,43 @@
 
 import json
 import orjson
+import structlog
 from abc import ABC, abstractmethod
+from enum import Enum
 from pydantic import BaseModel, Field, root_validator
 from rich import box
 from rich.console import ConsoleRenderable, Group, RenderableType, RichCast
 from rich.table import Table as RichTable
+from rich.tree import Tree
 from typing import (
     TYPE_CHECKING,
     Any,
     Dict,
     Iterable,
+    Iterator,
     List,
     Mapping,
-    Optional,
     Set,
     Type,
+    Union,
 )
 
 from kiara.defaults import SpecialValue
 from kiara.models.values.value import ORPHAN, Value, ValueMap
-from kiara.utils import dict_from_cli_args, orjson_dumps
+from kiara.utils.json import orjson_dumps
 
 if TYPE_CHECKING:
     from pyarrow import Table as ArrowTable
     from sqlalchemy.engine import Engine
 
+    from kiara.models.events.pipeline import PipelineState
+    from kiara.models.module.pipeline import PipelineStructure
     from kiara.models.values.value_schema import ValueSchema
 
 
+log = structlog.getLogger()
+
+
 class RenderConfig(BaseModel):
 
     render_format: str = Field(description="The output format.", default="terminal")
@@ -51,6 +60,8 @@ class OutputDetails(BaseModel):
                 data = [f"format={data}"]
 
         if isinstance(data, Iterable):
+            from kiara.utils.cli import dict_from_cli_args
+
             data = list(data)
             if len(data) == 1 and isinstance(data[0], str) and "=" not in data[0]:
                 data = [f"format={data[0]}"]
@@ -91,129 +102,11 @@ class OutputDetails(BaseModel):
         return result
 
 
-# def pretty_print_table(
-#     table: "Table",
-#     rows_head: typing.Optional[int] = None,
-#     rows_tail: typing.Optional[int] = None,
-#     max_row_height: typing.Optional[int] = None,
-#     max_cell_length: typing.Optional[int] = None,
-# ) -> RenderableType:
-#
-#     rich_table = RichTable(box=box.SIMPLE)
-#     for cn in table.column_names:
-#         rich_table.add_column(cn)
-#
-#     num_split_rows = 2
-#
-#     if rows_head is not None:
-#
-#         if rows_head < 0:
-#             rows_head = 0
-#
-#         if rows_head > table.num_rows:
-#             rows_head = table.num_rows
-#             rows_tail = None
-#             num_split_rows = 0
-#
-#         if rows_tail is not None:
-#             if rows_head + rows_tail >= table.num_rows:  # type: ignore
-#                 rows_head = table.num_rows
-#                 rows_tail = None
-#                 num_split_rows = 0
-#     else:
-#         num_split_rows = 0
-#
-#     if rows_head is not None:
-#         head = table.slice(0, rows_head)
-#         num_rows = rows_head
-#     else:
-#         head = table
-#         num_rows = table.num_rows
-#
-#     table_dict = head.to_pydict()
-#     for i in range(0, num_rows):
-#         row = []
-#         for cn in table.column_names:
-#             cell = table_dict[cn][i]
-#             cell_str = str(cell)
-#             if max_row_height and max_row_height > 0 and "\n" in cell_str:
-#                 lines = cell_str.split("\n")
-#                 if len(lines) > max_row_height:
-#                     if max_row_height == 1:
-#                         lines = lines[0:1]
-#                     else:
-#                         half = int(max_row_height / 2)
-#                         lines = lines[0:half] + [".."] + lines[-half:]
-#                 cell_str = "\n".join(lines)
-#
-#             if max_cell_length and max_cell_length > 0:
-#                 lines = []
-#                 for line in cell_str.split("\n"):
-#                     if len(line) > max_cell_length:
-#                         line = line[0:max_cell_length] + " ..."
-#                     else:
-#                         line = line
-#                     lines.append(line)
-#                 cell_str = "\n".join(lines)
-#
-#             row.append(cell_str)
-#
-#         rich_table.add_row(*row)
-#
-#     if num_split_rows:
-#         for i in range(0, num_split_rows):
-#             row = []
-#             for _ in table.column_names:
-#                 row.append("...")
-#             rich_table.add_row(*row)
-#
-#     if rows_head:
-#         if rows_tail is not None:
-#             if rows_tail < 0:
-#                 rows_tail = 0
-#
-#             tail = table.slice(table.num_rows - rows_tail)
-#             table_dict = tail.to_pydict()
-#             for i in range(0, num_rows):
-#
-#                 row = []
-#                 for cn in table.column_names:
-#
-#                     cell = table_dict[cn][i]
-#                     cell_str = str(cell)
-#
-#                     if max_row_height and max_row_height > 0 and "\n" in cell_str:
-#                         lines = cell_str.split("\n")
-#                         if len(lines) > max_row_height:
-#                             if max_row_height == 1:
-#                                 lines = lines[0:1]
-#                             else:
-#                                 half = int(len(lines) / 2)
-#                                 lines = lines[0:half] + [".."] + lines[-half:]
-#                         cell_str = "\n".join(lines)
-#
-#                     if max_cell_length and max_cell_length > 0:
-#                         lines = []
-#                         for line in cell_str.split("\n"):
-#
-#                             if len(line) > max_cell_length:
-#                                 line = line[0:(max_cell_length)] + " ..."
-#                             else:
-#                                 line = line
-#                             lines.append(line)
-#                         cell_str = "\n".join(lines)
-#
-#                     row.append(cell_str)
-#
-#                 rich_table.add_row(*row)
-#
-#     return rich_table
-
-
 class TabularWrap(ABC):
     def __init__(self):
-        self._num_rows: Optional[int] = None
-        self._column_names: Optional[Iterable[str]] = None
+        self._num_rows: Union[int, None] = None
+        self._column_names: Union[Iterable[str], None] = None
+        self._force_single_line: bool = True
 
     @property
     def num_rows(self) -> int:
@@ -236,25 +129,108 @@ class TabularWrap(ABC):
         pass
 
     @abstractmethod
-    def slice(self, offset: int = 0, length: Optional[int] = None) -> "TabularWrap":
+    def slice(self, offset: int = 0, length: Union[int, None] = None) -> "TabularWrap":
         pass
 
     @abstractmethod
     def to_pydict(self) -> Mapping:
         pass
 
-    def pretty_print(
+    def as_string(
         self,
-        rows_head: Optional[int] = None,
-        rows_tail: Optional[int] = None,
-        max_row_height: Optional[int] = None,
-        max_cell_length: Optional[int] = None,
-        show_table_header: bool = True,
-    ) -> RenderableType:
+        rows_head: Union[int, None] = None,
+        rows_tail: Union[int, None] = None,
+        max_row_height: Union[int, None] = None,
+        max_cell_length: Union[int, None] = None,
+    ):
 
-        rich_table = RichTable(box=box.SIMPLE, show_header=show_table_header)
-        for cn in self.retrieve_column_names():
-            rich_table.add_column(cn)
+        table_str = ""
+        for cn in self.column_names:
+            table_str = f"{table_str}{cn}\t"
+        table_str = f"{table_str}\n"
+
+        for data in self.prepare_table_data(
+            return_column_names=False,
+            rows_head=rows_head,
+            rows_tail=rows_tail,
+            max_row_height=max_row_height,
+            max_cell_length=max_cell_length,
+        ):
+            for cell in data:
+                table_str = f"{table_str}{cell}\t"
+            table_str = f"{table_str}\n"
+
+        return table_str
+
+    def as_html(
+        self,
+        rows_head: Union[int, None] = None,
+        rows_tail: Union[int, None] = None,
+        max_row_height: Union[int, None] = None,
+        max_cell_length: Union[int, None] = None,
+    ) -> str:
+
+        table_str = "<table><tr>"
+        for cn in self.column_names:
+            table_str = f"{table_str}<th>{cn}</th>"
+        table_str = f"{table_str}</tr>"
+
+        for data in self.prepare_table_data(
+            return_column_names=False,
+            rows_head=rows_head,
+            rows_tail=rows_tail,
+            max_row_height=max_row_height,
+            max_cell_length=max_cell_length,
+        ):
+            table_str = f"{table_str}<tr>"
+            for cell in data:
+                table_str = f"{table_str}<td>{cell}</td>"
+            table_str = f"{table_str}</tr>"
+        table_str = f"{table_str}</table>"
+        return table_str
+
+    def as_terminal_renderable(
+        self,
+        rows_head: Union[int, None] = None,
+        rows_tail: Union[int, None] = None,
+        max_row_height: Union[int, None] = None,
+        max_cell_length: Union[int, None] = None,
+        show_table_header: bool = True,
+    ) -> RichTable:
+
+        rich_table = RichTable(show_header=show_table_header, box=box.SIMPLE)
+        if max_row_height == 1:
+            overflow = "ignore"
+        else:
+            overflow = "ellipsis"
+
+        for cn in self.column_names:
+            rich_table.add_column(cn, overflow=overflow)  # type: ignore
+
+        data = self.prepare_table_data(
+            return_column_names=False,
+            rows_head=rows_head,
+            rows_tail=rows_tail,
+            max_row_height=max_row_height,
+            max_cell_length=max_cell_length,
+        )
+
+        for row in data:
+            rich_table.add_row(*row)
+
+        return rich_table
+
+    def prepare_table_data(
+        self,
+        return_column_names: bool = False,
+        rows_head: Union[int, None] = None,
+        rows_tail: Union[int, None] = None,
+        max_row_height: Union[int, None] = None,
+        max_cell_length: Union[int, None] = None,
+    ) -> Iterator[Iterable[Any]]:
+
+        if return_column_names:
+            yield self.column_names
 
         num_split_rows = 2
 
@@ -263,14 +239,14 @@ class TabularWrap(ABC):
             if rows_head < 0:
                 rows_head = 0
 
-            if rows_head > self.retrieve_number_of_rows():
-                rows_head = self.retrieve_number_of_rows()
+            if rows_head > self.num_rows:
+                rows_head = self.num_rows
                 rows_tail = None
                 num_split_rows = 0
 
             if rows_tail is not None:
                 if rows_head + rows_tail >= self.num_rows:  # type: ignore
-                    rows_head = self.retrieve_number_of_rows()
+                    rows_head = self.num_rows
                     rows_tail = None
                     num_split_rows = 0
         else:
@@ -281,12 +257,12 @@ class TabularWrap(ABC):
             num_rows = rows_head
         else:
             head = self
-            num_rows = self.retrieve_number_of_rows()
+            num_rows = self.num_rows
 
         table_dict = head.to_pydict()
         for i in range(0, num_rows):
             row = []
-            for cn in self.retrieve_column_names():
+            for cn in self.column_names:
                 cell = table_dict[cn][i]
                 cell_str = str(cell)
                 if max_row_height and max_row_height > 0 and "\n" in cell_str:
@@ -311,26 +287,26 @@ class TabularWrap(ABC):
 
                 row.append(cell_str)
 
-            rich_table.add_row(*row)
+            yield row
 
         if num_split_rows:
             for i in range(0, num_split_rows):
                 row = []
-                for _ in self.retrieve_column_names():
+                for _ in self.column_names:
                     row.append("...")
-                rich_table.add_row(*row)
+                yield row
 
         if rows_head:
             if rows_tail is not None:
                 if rows_tail < 0:
                     rows_tail = 0
 
-                tail = self.slice(self.retrieve_number_of_rows() - rows_tail)
+                tail = self.slice(self.num_rows - rows_tail)
                 table_dict = tail.to_pydict()
                 for i in range(0, num_rows):
 
                     row = []
-                    for cn in self.retrieve_column_names():
+                    for cn in self.column_names:
 
                         cell = table_dict[cn][i]
                         cell_str = str(cell)
@@ -358,9 +334,9 @@ class TabularWrap(ABC):
 
                         row.append(cell_str)
 
-                    rich_table.add_row(*row)
+                    yield row
 
-        return rich_table
+        return
 
 
 class ArrowTabularWrap(TabularWrap):
@@ -374,7 +350,7 @@ class ArrowTabularWrap(TabularWrap):
     def retrieve_number_of_rows(self) -> int:
         return self._table.num_rows
 
-    def slice(self, offset: int = 0, length: Optional[int] = None):
+    def slice(self, offset: int = 0, length: Union[int, None] = None):
         return self._table.slice(offset=offset, length=length)
 
     def to_pydict(self) -> Mapping:
@@ -382,20 +358,23 @@ class ArrowTabularWrap(TabularWrap):
 
 
 class DictTabularWrap(TabularWrap):
-    def __init__(self, data: Mapping[str, Any]):
+    def __init__(self, data: Mapping[str, List[Any]]):
 
-        self._data: Mapping[str, Any] = data
+        self._data: Mapping[str, List[Any]] = data
+        # TODO: assert all rows are equal length
+        super().__init__()
 
     def retrieve_number_of_rows(self) -> int:
-        return len(self._data)
+        key = next(iter(self._data.keys()))
+        return len(self._data[key])
 
     def retrieve_column_names(self) -> Iterable[str]:
         return self._data.keys()
 
-    def to_pydict(self) -> Mapping:
+    def to_pydict(self) -> Mapping[str, List[Any]]:
         return self._data
 
-    def slice(self, offset: int = 0, length: Optional[int] = None) -> "TabularWrap":
+    def slice(self, offset: int = 0, length: Union[int, None] = None) -> "TabularWrap":
 
         result = {}
         start = None
@@ -441,7 +420,7 @@ class SqliteTabularWrap(TabularWrap):
         result = [column["name"] for column in columns]
         return result
 
-    def slice(self, offset: int = 0, length: Optional[int] = None) -> "TabularWrap":
+    def slice(self, offset: int = 0, length: Union[int, None] = None) -> "TabularWrap":
 
         from sqlalchemy import text
 
@@ -523,47 +502,16 @@ def create_table_from_base_model_cls(model_cls: Type[BaseModel]):
     return table
 
 
-# def create_table_from_config_class(
-#     config_cls: typing.Type["KiaraModuleConfig"],
-#     remove_pipeline_config: bool = False,
-# ) -> Table:
-#
-#     table = Table(box=box.HORIZONTALS, show_header=False)
-#     table.add_column("Field name", style="i")
-#     table.add_column("Type")
-#     table.add_column("Description")
-#     flat_models = get_flat_models_from_model(config_cls)
-#     model_name_map = get_model_name_map(flat_models)
-#     m_schema, _, _ = model_process_schema(config_cls, model_name_map=model_name_map)
-#     fields = m_schema["properties"]
-#
-#     for alias, details in fields.items():
-#         if remove_pipeline_config and alias in [
-#             "steps",
-#             "input_aliases",
-#             "output_aliases",
-#             "doc",
-#         ]:
-#             continue
-#
-#         type_str = "-- n/a --"
-#         if "type" in details.keys():
-#             type_str = details["type"]
-#         table.add_row(alias, type_str, details.get("description", "-- n/a --"))
-#
-#     return table
-
-
 def create_table_from_field_schemas(
+    fields: Mapping[str, "ValueSchema"],
     _add_default: bool = True,
     _add_required: bool = True,
     _show_header: bool = False,
-    _constants: Optional[Mapping[str, Any]] = None,
-    **fields: "ValueSchema",
+    _constants: Union[Mapping[str, Any], None] = None,
 ) -> RichTable:
 
     table = RichTable(box=box.SIMPLE, show_header=_show_header)
-    table.add_column("field name", style="i")
+    table.add_column("field name", style="i", overflow="fold")
     table.add_column("type")
     table.add_column("description")
 
@@ -574,7 +522,6 @@ def create_table_from_field_schemas(
             table.add_column("Default / Constant")
         else:
             table.add_column("Default")
-
     for field_name, schema in fields.items():
 
         row: List[RenderableType] = [field_name, schema.type, schema.doc]
@@ -614,20 +561,27 @@ def create_table_from_field_schemas(
 
 
 def create_value_map_status_renderable(
-    inputs: ValueMap, render_config: Optional[Mapping[str, Any]] = None
+    inputs: ValueMap,
+    render_config: Union[Mapping[str, Any], None] = None,
+    fields: Union[None, Iterable[str]] = None,
 ) -> RichTable:
 
     if render_config is None:
         render_config = {}
 
+    show_description: bool = render_config.get("show_description", True)
+    show_type: bool = render_config.get("show_type", True)
     show_required: bool = render_config.get("show_required", True)
     show_default: bool = render_config.get("show_default", True)
+    show_value_ids: bool = render_config.get("show_value_ids", False)
 
     table = RichTable(box=box.SIMPLE, show_header=True)
     table.add_column("field name", style="i")
     table.add_column("status", style="b")
-    table.add_column("type")
-    table.add_column("description")
+    if show_type:
+        table.add_column("type")
+    if show_description:
+        table.add_column("description")
 
     if show_required:
         table.add_column("required")
@@ -635,9 +589,24 @@ def create_value_map_status_renderable(
     if show_default:
         table.add_column("default")
 
+    if show_value_ids:
+        table.add_column("value id", overflow="fold")
+
     invalid = inputs.check_invalid()
 
-    for field_name, value in inputs.items():
+    if fields:
+        field_order = fields
+    else:
+        field_order = sorted(inputs.keys())
+
+    for field_name in field_order:
+
+        value = inputs.get(field_name, None)
+        if value is None:
+            log.debug(
+                "ignore.field", field_name=field_name, available_fields=inputs.keys()
+            )
+            continue
 
         row: List[RenderableType] = [field_name]
 
@@ -648,7 +617,11 @@ def create_value_map_status_renderable(
 
         value_schema = inputs.values_schema[field_name]
 
-        row.extend([value_schema.type, value_schema.doc.description])
+        if show_type:
+            row.append(value_schema.type)
+
+        if show_description:
+            row.append(value_schema.doc.description)
 
         if show_required:
             req = value_schema.is_required()
@@ -679,6 +652,9 @@ def create_value_map_status_renderable(
 
             row.append(default_str)
 
+        if show_value_ids:
+            row.append(str(inputs.get_value_obj(field_name=field_name).value_id))
+
         table.add_row(*row)
 
     return table
@@ -686,8 +662,8 @@ def create_value_map_status_renderable(
 
 def create_table_from_model_object(
     model: BaseModel,
-    render_config: Optional[Mapping[str, Any]] = None,
-    exclude_fields: Optional[Set[str]] = None,
+    render_config: Union[Mapping[str, Any], None] = None,
+    exclude_fields: Union[Set[str], None] = None,
 ):
 
     model_cls = model.__class__
@@ -703,7 +679,7 @@ def create_table_from_model_object(
     for field_name, field in sorted(model_cls.__fields__.items()):
         if exclude_fields and field_name in exclude_fields:
             continue
-        row = [field_name]
+        row: List[RenderableType] = [field_name]
 
         p = props.get(field_name, None)
         p_type = None
@@ -725,7 +701,9 @@ def create_table_from_model_object(
     return table
 
 
-def extract_renderable(item: Any, render_config: Optional[Mapping[str, Any]] = None):
+def extract_renderable(
+    item: Any, render_config: Union[Mapping[str, Any], None] = None
+) -> RenderableType:
     """Try to automatically find and extract or create an object that is renderable by the 'rich' library."""
 
     if render_config is None:
@@ -765,12 +743,14 @@ def extract_renderable(item: Any, render_config: Optional[Mapping[str, Any]] = N
             _all.append(extract_renderable(i))
         rg = Group(*_all)
         return rg
+    elif isinstance(item, Enum):
+        return item.value
     else:
         return str(item)
 
 
 def create_renderable_from_values(
-    values: Mapping[str, "Value"], config: Optional[Mapping[str, Any]] = None
+    values: Mapping[str, "Value"], config: Union[Mapping[str, Any], None] = None
 ) -> RenderableType:
     """Create a renderable for this module configuration."""
 
@@ -822,3 +802,91 @@ def create_renderable_from_values(
         table.add_row(*row)
 
     return table
+
+
+def create_pipeline_steps_tree(
+    pipeline_structure: "PipelineStructure", pipeline_details: "PipelineState"
+) -> Tree:
+
+    from kiara.models.module.pipeline import StepStatus
+
+    steps = Tree("steps")
+
+    for idx, stage in enumerate(pipeline_structure.processing_stages, start=1):
+        stage_node = steps.add(f"stage: [i]{idx}[/i]")
+        for step_id in sorted(stage):
+            step_node = stage_node.add(f"step: [i]{step_id}[/i]")
+            step_details = pipeline_details.step_states[step_id]
+            status = step_details.status
+            if status is StepStatus.INPUTS_READY:
+                step_node.add("status: [yellow]inputs ready[/yellow]")
+            elif status is StepStatus.RESULTS_READY:
+                step_node.add("status: [green]results ready[/green]")
+            else:
+                invalid_node = step_node.add("status: [red]inputs invalid[/red]")
+                invalid = step_details.invalid_details
+                for k, v in invalid.items():
+                    invalid_node.add(f"[i]{k}[/i]: {v}")
+
+    return steps
+
+
+def create_recursive_table_from_model_object(
+    model: BaseModel,
+    render_config: Union[Mapping[str, Any], None] = None,
+):
+
+    if render_config is None:
+        render_config = {}
+
+    show_lines = render_config.get("show_lines", True)
+    show_header = render_config.get("show_header", True)
+    model_cls = model.__class__
+
+    table = RichTable(box=box.SIMPLE, show_lines=show_lines, show_header=show_header)
+    table.add_column("Field")
+    table.add_column("Value")
+
+    props = model_cls.schema().get("properties", {})
+
+    for field_name in sorted(model_cls.__fields__.keys()):
+
+        data = getattr(model, field_name)
+        p = props.get(field_name, None)
+        p_type = None
+        if p is not None:
+            p_type = p.get("type", None)
+            # TODO: check 'anyOf' keys
+
+        if p_type is not None:
+            p_type = f"[i]{p_type}[/i]"
+
+        desc = p.get("description", None)
+
+        if not isinstance(data, BaseModel):
+            data_renderable = extract_renderable(data, render_config=render_config)
+            sub_model = None
+        else:
+            sub_model = create_recursive_table_from_model_object(
+                data, render_config={"show_lines": True, "show_header": False}
+            )
+            data_renderable = None
+
+        group = []
+
+        if data_renderable:
+            group.append(data_renderable)
+            group.append("")
+        if desc:
+            group.append(f"[i]{desc}[/i]")
+
+        if sub_model:
+            group.append(sub_model)
+
+        if p_type:
+            field_name = f"[b i]{field_name}[/b i] ([i]{p_type}[/i])"
+        else:
+            field_name = f"[b i]{field_name}[/b i]"
+        table.add_row(field_name, Group(*group))
+
+    return table
